% % Denne funksjonen komprimere bilde-arrays etter [Reinhard05]
% 
% % Parameter   Description             Initial value       Operating range
% % input_array Inngangsbilde           --                  --
% % m_inn       Contrast                0.0                 [-1.0  ;  1.0]
% % f_inn_input Intensity               0.0                 [-18.0 ; 18.0]
% % c           Chromatic adaptation    0.0                 [0.0   ;  1.0]
% % a           Light adaptation        1.0                 [0.0   ;  1.0]

function out = PhPh_array(input_array, m_inn, f_inn_input, c, a, output_file)

% Leser inn pikselverdiene fra bildet
% read_rle_rgbe() er laget av Lawrence A. Taplin og er tilgjengelig p
% webadressen http://www.cis.rit.edu/mcsl/icam/hdr/read_rle_rgbe.m
I = input_array;
% Skalerer inngangssignalet til 18-bits heltall
I = I/max(max(max(I)));
Maks_inngang = 2^18;
I = round(I*Maks_inngang);

% Finner dimensjonene p inngangs-bildet
[height width dim] = size(I);

% Regner ut luminansen i hvert piksel samt finner minimumsveriden og den
% logaritmiske gjennomsnittsverdien
Lmin = 9999999;
Llav = 0;
Lav_test  = 0;
for(i=1:1:height)
    for(j=1:1:width)
           Luminance(i,j) = floor(0.2109*I(i,j,1) + 0.7188*I(i,j,2) + 0.0723*I(i,j,3));           
        if(Luminance(i,j) > 0)
            Llav = Llav + log2(Luminance(i,j));
        end
        if((Luminance(i,j) ~= 0)  && (Luminance(i,j)<Lmin))
            Lmin = Luminance(i,j);
        end
    end
end
Llmin = log2(Lmin);
Llav = Llav/(height*width);
Lav  = 2^(Llav);

% regner ut noen kompresjonsparametre
Lmax = max(max(Luminance));
Llmax = log2(Lmax);

f_inn = (Llav*1.375) + f_inn_input;

f=2^(f_inn);
k = (Llmax-Llav)/(Llmax-Llmin);
m = 0.3 + ((1.375/2)*k^(1.4)) + (m_inn/2)

% Passer p at m er innenfor lovlig omrde
if(m>1)
    m=1;
end

if(m<0)
    m=0;
end

if(Llmin<0) 
    Llmin=0;
end

% regner ut den globale adapsjonen
mean_intensity = mean(mean(I));
for(d=1:1:dim)
    Ia_global(d) = c*mean_intensity(d) + (1-c)*Lav;
end

% Tar tiden p selve dynamikkompresjonen
tic     

% Gr igjennom bildet og komprimerer piksel for piksel
for(i=1:1:height)
    for(j=1:1:width)
        for(d=1:1:dim)
            if(I(i,j,d)==0)
%               Setter verdien til 0 om pikselverdien er null
%               (for  slippe  dele p null i kompresjonen)
                out(i,j,d) = I(i,j,d);
            else
%               regner ut fargen ut fra kromatisk adapsjon
            Ia_local = c*I(i,j,d) + (1-c)*Luminance(i,j); 
%               Regner ut adapsjonsniv ut fra vekting mellom 
%               Lokal og global adapsjon   
            Ia = a*Ia_local + (1-a)*Ia_global(d);
%               Regner pikselets ut kompresjonsfaktor
            sigma = (f*Ia)^m;
            sigma_vektor(i,j,d) = sigma;
%               Komprimerer
            out(i,j,d) = (I(i,j,d)/(I(i,j,d)+sigma));            
            end
        end      
    end
end

% leser ut tiden det tok  komprimere bildet
toc

% Viser frem bildet p skjermen
image(out/max(max(max(out))));

% Lagrer bildet som bitmapfil
imwrite(out/max(max(max(out))), output_file, 'bmp');
