---------------------------------------
-- This module calculates the luminance of the input RGB-signal
---------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

---------------------------------------
-- Defining the Entity
---------------------------------------
entity luminance is
  generic(NBITS  : natural := 18         -- # bits
          );
    port (red   : in  std_logic_vector (NBITS-1 downto 0);
          green : in  std_logic_vector (NBITS-1 downto 0);
          blue  : in  std_logic_vector (NBITS-1 downto 0);
          output: out std_logic_vector (NBITS-1 downto 0)
          );
end entity luminance;

---------------------------------------
-- Behavioral Architecture
---------------------------------------
architecture behavior of luminance is

-- Defining components    
component add_g is
  generic(left : natural := 21         -- top bit
          );
    port (a    : in  std_logic_vector (left downto 0);
          b    : in  std_logic_vector (left downto 0);
          cin  : in  std_logic;
          sum  : out std_logic_vector (left downto 0);
          cout : out std_logic);
end component add_g;    
    
    signal red_1    : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal red_2    : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal red_3    : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal red_4    : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal red_5    : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal red_6    : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal red_7    : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal green_1  : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal green_2  : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal green_3  : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal green_4  : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal green_5  : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal green_6  : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal green_7  : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal blue_1   : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal blue_2   : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal blue_3   : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal blue_4   : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal blue_5   : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal temp     : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    signal output_temp : std_logic_vector(NBITS-1+4 downto 0) := (others=>'0');
    
    signal cout_0   : std_logic;
    signal cout_1   : std_logic;
    signal cout_2   : std_logic;
    signal cout_3   : std_logic;
    signal cout_4   : std_logic;
    signal cout_5   : std_logic;
    signal cout_6   : std_logic;
    signal cout_7   : std_logic;
    signal cout_8   : std_logic;
    signal cout_9   : std_logic;
    
begin  -- behavior

-- Initializing components

   add1:   add_g   port map(red_1, red_2, '0', red_5, cout_1);
   add2:   add_g   port map(red_3, red_4, '0', red_6, cout_2);
   add3:   add_g   port map(red_5, red_6, '0', red_7, cout_3);
   
   add4:   add_g   port map(green_1, green_2, '0', green_5, cout_4);
   add5:   add_g   port map(green_3, green_4, '0', green_6, cout_5);
   add6:   add_g   port map(green_5, green_6, '0', green_7, cout_6);
   
   add7:   add_g   port map(blue_1, blue_2, '0', blue_4, cout_7);
   add8:   add_g   port map(blue_3, blue_4, '0', blue_5, cout_8);
   
   add9:   add_g   port map(red_7, green_7, '0', temp, cout_9);
   add0:   add_g   port map(temp, blue_5, '0', output_temp, cout_0);

-- Assigning internal temporary signals   
   red_1   <= ("000" & red(NBITS-1 downto 0) & "0");
   red_2   <= ("0000" & red(NBITS-1 downto 0));
   red_3   <= ("000000" & red(NBITS-1 downto 2));
   red_4   <= ("0000000" & red(NBITS-1 downto 3));
   
   green_1 <= ("0" & green(NBITS-1 downto 0) & "000");
   green_2 <= ("000" & green(NBITS-1 downto 0) & "0");
   green_3 <= ("0000" & green(NBITS-1 downto 0));
   green_4 <= ("00000" & green(NBITS-1 downto 1));
   
   blue_1  <= ("0000" & blue(NBITS-1 downto 0));
   blue_2  <= ("0000000" & blue(NBITS-1 downto 3));
   blue_3  <= ("000000000" & blue(NBITS-1 downto 5));
   
-- Assigning the output   
   output  <= output_temp(NBITS-1+4 downto 4);
    
end architecture behavior;  -- of luminance