---------------------------------------
-- This module calculates the maksimum value of a stream of values
-- until reset.
-- This module is fully generic
---------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

---------------------------------------
-- Entity
---------------------------------------
entity maksimum is
  generic(NBITS  : natural := 18         -- # bits
          );
    port (input : in  std_logic_vector (NBITS-1 downto 0);
          clk   : in  std_logic;    
          reset : in  std_logic;
          output: out std_logic_vector (NBITS-1 downto 0)
          );
end entity maksimum;

---------------------------------------
-- Architecture
---------------------------------------
architecture behavior of maksimum is
    signal internal : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
begin  -- behavior

-- Checking if the input is a higher value than the internal register.
-- If so, then overwrite the internal register with the input value.    
process (clk, internal)
   begin
   if(rising_edge(clk)) then
       if(reset='1') then            -- Active high reset
           internal <= (others =>'0');
       else
          if(input > internal) then
              internal <= input;
          end if;
       end if;
   end if;
   output <= internal;                
end process;
end architecture behavior;  -- of maksimum