---------------------------------------
-- This module calculates the compression parameters.
-- This module is compatible with both of the architectures.
---------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

---------------------------------------
-- Defining the Entity
---------------------------------------
entity param is
  generic(NBITS  : natural := 18         -- # bits
          );
    port (clk         : in  std_logic;
          reset       : in  std_logic;
          comp_phase  : in  std_logic;
          go          : in  std_logic;
          
          R           : in  std_logic_vector (NBITS-1 downto 0);
          G           : in  std_logic_vector (NBITS-1 downto 0);
          B           : in  std_logic_vector (NBITS-1 downto 0);
          R_enable    : in  std_logic;
          G_enable    : in  std_logic;
          B_enable    : in  std_logic;
    			 Input_Lum   : in  std_logic_vector (NBITS-1 downto 0);  
    			 Lum_enable  : in  std_logic;                  
          m_in        : in  std_logic_vector (12 downto 0);   
          cntrl_m     : in  std_logic;       
          f_user      : in  std_logic_vector (NBITS-1 downto 0);
          cntrl_f     : in  std_logic;          
          
          Div_ans     : in  std_logic_vector (8-1 downto 0);
          Sig_ans     : in  std_logic_vector (NBITS-1 downto 0);
          Ia_in       : in  std_logic_vector (NBITS-1 downto 0);
          select_sig  : out std_logic;
          select_div  : out std_logic;                          
          select_Ia   : out std_logic;
          Ia_out_1    : out std_logic_vector (NBITS-1 downto 0);
          Ia_out_2    : out std_logic_vector (NBITS-1 downto 0);
          Sigma_1     : out std_logic_vector (NBITS-1 downto 0);
          Sigma_2     : out std_logic_vector (NBITS-1 downto 0);          
          Div_1       : out std_logic_vector (NBITS-1 downto 0);
          Div_2       : out std_logic_vector (NBITS-1 downto 0);
            
          Ia_g_R_out  : out std_logic_vector (NBITS-1 downto 0);
          Ia_g_G_out  : out std_logic_vector (NBITS-1 downto 0);
          Ia_g_B_out  : out std_logic_vector (NBITS-1 downto 0);          
          m_out       : out std_logic_vector (12 downto 0);  
          f           : out std_logic_vector (NBITS-1 downto 0)
          );
end entity param;

---------------------------------------
-- Behavioral Architecture.
---------------------------------------
architecture behavior of param is   

-- Defining the components
component add_g is
  generic(left : natural := NBITS-1         -- top bit
          );
    port (a    : in  std_logic_vector (left downto 0);
          b    : in  std_logic_vector (left downto 0);
          cin  : in  std_logic;
          sum  : out std_logic_vector (left downto 0);
          cout : out std_logic);
end component add_g;
   
component avg is
    generic(NBITS : natural :=NBITS);
    port(
       input   :   in std_logic_vector(NBITS-1 downto 0);
       enable  :   in std_logic;
       reset   :   in std_logic;
       clk     :   in std_logic;
       output  :   out std_logic_vector(NBITS-1 downto 0));
end component avg;

component maksimum is
  generic(NBITS  : natural := NBITS         -- # bits
          );
    port (input : in  std_logic_vector (NBITS-1 downto 0);
          clk   : in  std_logic;    
          reset : in  std_logic;
          output: out std_logic_vector (NBITS-1 downto 0)
          );
end component maksimum;

component minimum is
  generic(NBITS  : natural := NBITS         -- # bits
          );
    port (input : in  std_logic_vector (NBITS-1 downto 0);
          clk   : in  std_logic;    
          reset : in  std_logic;
          output: out std_logic_vector (NBITS-1 downto 0)
          );
end component minimum;

component f_inn is
	 generic( NBITS : natural := NBITS
	 );
    port(       
       input            :   in  std_logic_vector(NBITS-1 downto 0);
       f_user           :   in  std_logic_vector(NBITS-1 downto 0);
       cntrl_f          :   in  std_logic;
       output           :   out std_logic_vector(NBITS-1 downto 0)
       );
end component f_inn;

component Logarithm_Abed03_3region is
    port(
       input   :   in std_logic_vector(17 downto 0);
       zero_in :   out std_logic;
       output  :   out std_logic_vector(17 downto 0)
       );
end component Logarithm_Abed03_3region;

component addsub is
generic(NBITS : natural := NBITS
   );
   port 
      (x       : in std_logic_vector(NBITS-1 downto 0);
       y       : in std_logic_vector(NBITS-1 downto 0);
       control : in std_logic;
       d_in    : in std_logic;
       z       : out std_logic_vector(NBITS-1 downto 0);
       d_out   : out std_logic
      );
end component addsub;

component antilogarithm is
    port(
       input   :   in std_logic_vector(17 downto 0);
       output  :   out std_logic_vector(17 downto 0)
       );
end component antilogarithm;

-- Defining the internal signals

    signal Log_Lum          : std_logic_vector(NBITS-1 downto 0);
    signal Log_Lum_pipe     : std_logic_vector(NBITS-1 downto 0);
    signal R_avg            : std_logic_vector(NBITS-1 downto 0);
    signal G_avg            : std_logic_vector(NBITS-1 downto 0);
    signal B_avg            : std_logic_vector(NBITS-1 downto 0);
    signal Lum_avg          : std_logic_vector(NBITS-1 downto 0);
    signal Llum_avg         : std_logic_vector(NBITS-1 downto 0);
    signal Maks_Lum         : std_logic_vector(NBITS-1 downto 0);
    signal Min_Lum          : std_logic_vector(NBITS-1 downto 0);
    signal f_inn_output     : std_logic_vector(NBITS-1 downto 0);
    signal input_f          : std_logic_vector(NBITS-1 downto 0);
    signal f_user_input     : std_logic_vector(NBITS-1 downto 0);
    
    signal k_temp_1         : std_logic_vector(NBITS-1 downto 0);
    signal k_temp_2         : std_logic_vector(NBITS-1 downto 0);
    signal k_temp           : std_logic_vector(NBITS-1 downto 0);
    signal k                : std_logic_vector(8-1 downto 0);
    signal m                : std_logic_vector(12 downto 0);
    signal input_Log        : std_logic_vector(NBITS-1 downto 0);
    signal add_m_inn        : std_logic_vector(12 downto 0);
    
    signal lum_zero         : std_logic;
    signal cout_1           : std_logic;
    signal cout_2           : std_logic;
    signal cout_3           : std_logic;
    signal cntrl_f_internal : std_logic;
    
    signal counter          : std_logic_vector(2 downto 0);
    signal calc_k           : std_logic;
    signal store_k          : std_logic;
    signal store_f          : std_logic;
    signal store_m          : std_logic;
    signal select_Log       : std_logic;
    signal reset_avg        : std_logic;
    
    signal Lum_enable_pipe:std_logic;
    
begin  -- behavior

-- Initializing the components
    
    Log         :   Logarithm_Abed03_3region   port map(input_Log, lum_zero, Log_Lum);
    Maks        :   maksimum                   port map(Log_Lum, clk, reset_avg, Maks_Lum);
    Min         :   minimum                    port map(Log_Lum, clk, reset_avg, Min_Lum);
    Avg_r       :   avg                        port map(R, R_enable, reset_avg, clk, R_avg);
    Avg_g       :   avg                        port map(G, G_enable, reset_avg, clk, G_avg);
    Avg_b       :   avg                        port map(B, B_enable, reset_avg, clk, B_avg);
    Avg_Llum    :   avg                        port map(Log_Lum_pipe, Lum_enable_pipe, reset_avg, clk, Llum_avg);
    f_parameter :   f_inn                      port map(input_f, f_user_input, cntrl_f_internal, f_inn_output);
    Avg_exp_Lum :   antilogarithm              port map(Llum_avg, Lum_avg);
    
    sub_k_1     :   addsub                     port map(Maks_Lum, Llum_avg, '1', '0', k_temp_1, cout_1); -- sjekk om cntrl=1 eller 0 gir sub
    sub_k_2     :   addsub                     port map(Maks_Lum, Min_Lum, '1', '0', k_temp_2, cout_2); -- sjekk om cntrl=1 eller 0 gir sub
    add_m       :   add_g   generic map(12)    port map(add_m_inn, "0100110011001", '0', m, cout_3); -- add 0.3 -- 10011

-- Assigning some of the internal signals

    add_m_inn  <=   f_inn_output(13 downto 1); --"0" & 
    Sigma_2    <=   "000010101111010000"; 
    calc_k     <=   ((NOT counter(2)) AND (counter(1)) AND (NOT counter(0))) OR 
                    ((NOT counter(2)) AND (counter(1))     AND (counter(0)));
    store_k    <=   (counter(2)) AND (NOT counter(1))     AND (NOT counter(0));
    select_Log <=   ((counter(2)) AND (NOT counter(1)) AND (counter(0)));
    store_f    <=   (NOT counter(2)) AND (counter(1)) AND (NOT counter(0));
    store_m    <=   (counter(2)) AND (counter(1)) AND (counter(0));
    select_sig <=   (counter(2)) AND (counter(1)) AND (NOT counter(0));
    select_div <=   calc_k;
    select_Ia  <=   ((NOT counter(2)) AND (counter(1)) AND (NOT counter(0))) OR
                    ((NOT counter(2)) AND (counter(1)) AND (counter(0))) OR
                    ((counter(2)) AND (NOT counter(1)) AND (NOT counter(0)));                    
    reset_avg  <=   reset OR store_m OR (NOT go);
    
-- Sets the Ia_out_2 output to the value of Lum_avg with pipelining
Lum_avg_pipe: process(clk)
begin
    if(rising_edge(clk)) then
        if(counter = "001") then
            Ia_out_2   <=   Lum_avg;
        end if;
    end if;
end process Lum_avg_pipe;

-- An internal counter that keeps track of the parameter-calculation-phase    
counting: process(clk)
begin
    if(rising_edge(clk)) then
        if(comp_phase = '1' OR reset = '1') then
            counter <= (others=>'0');
        else
            counter <= counter +1;
        end if;
    end if;
end process counting;     

-- Calculates k with the help from the division-circuit in the signal-path.                               
k_process: process(calc_k, k_temp_1, k_temp_2)
begin
    if(calc_k = '1') then
        Div_1   <= k_temp_1;
        Div_2   <= k_temp_2;
    else
        Div_2   <= (others => '0');
        Div_1   <= (others => '0');
    end if;
end process k_process;

-- Stores k when it is calculated by the signal path.
k_process_2: process(clk)
begin
    if(rising_edge(clk)) then
       if(store_k = '1') then
           k       <=   Div_ans;
      end if;          
   end if;
end process k_process_2;

-- Calculates and stores the parameter m.
m_process: process(clk)
begin
    if(rising_edge(clk)) then
        if(comp_phase = '0' AND select_log='1') then
            Sigma_1   <= Log_Lum;
        end if;
        if(comp_phase = '0') then
            k_temp    <= Sig_ans;
        end if;   
        if(store_m='1') then
           m_out <= m;
        end if;
    end if;        
end process m_process;

-- stores the parameter f before the f_inn module gets occupied.
f_store_process: process(clk)
begin
    if(rising_edge(clk)) then
        if(store_f='1') then
            f <= f_inn_output;
        end if;
    end if;
end process f_store_process;

-- Selects what inputs the f_inn module gets.
f_inn_process: process(Llum_avg, f_user, store_f)
begin
    if(store_f = '1') then
       input_f            <= Llum_avg;
       f_user_input       <= f_user;
       cntrl_f_internal   <= cntrl_f;
    else
       cntrl_f_internal   <= cntrl_m;
       input_f            <= k_temp(15 downto 0) & "00";
       f_user_input       <= ("0000" & m_in & "0");
    end if;
end process f_inn_process;

-- Selects the input-signal to the logarithm-module.
Log_mux: process(input_Lum, select_log, k)   
begin
   if(select_log = '0') then
      input_Log   <= input_Lum;
   else
      input_Log   <= ("0000000000" & k);
   end if;
end process Log_mux;

-- Selects the input-signal to the Ia_global module.
Ia_global_process: process(counter, R_avg, G_avg, B_avg, Ia_in)   
begin
        case(counter) is
            when "010" => Ia_out_1    <= R_avg;
                          Ia_g_R_out  <= Ia_in;
            when "011" => Ia_out_1    <= G_avg;
                          Ia_g_G_out  <= Ia_in;
            when "100" => Ia_out_1    <= B_avg;
                          Ia_g_B_out  <= Ia_in;
            when others => Ia_out_1   <= (others => '0');
        end case;
end process Ia_global_process;

-- Pipelinesteps for increased performance.
pipeline: process(clk)
begin
    if(rising_edge(clk)) then
        Log_Lum_pipe   <=   Log_Lum;
        Lum_enable_pipe<=   Lum_enable;
    end if;
end process;

end architecture behavior; 