---------------------------------------
-- This module represents the entire signal-path of the pixelvalues.
-- This module is not fully generic.
---------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

---------------------------------------
-- Defines the Entity
---------------------------------------
entity signal_pipe is
  generic(NBITS  : natural := 18         -- # bits
          );
    port (Input_RGB   : in  std_logic_vector (NBITS-1 downto 0);
    			 Input_Lum   : in  std_logic_vector (NBITS-1 downto 0);          
          f           : in  std_logic_vector (NBITS-1 downto 0);
          a           : in  std_logic_vector (2 downto 0);
          c           : in  std_logic_vector (2 downto 0);          
          m           : in  std_logic_vector (12 downto 0);          
          Ia_global   : in  std_logic_vector (NBITS-1 downto 0);
          clk         : in  std_logic;
          Ia_in_1     : in  std_logic_vector (NBITS-1 downto 0);
          Ia_in_2     : in  std_logic_vector (NBITS-1 downto 0);          
          Div_1       : in  std_logic_vector (NBITS-1 downto 0);
          Div_2       : in  std_logic_vector (NBITS-1 downto 0);
          select_div  : in  std_logic;
          select_Ia   : in  std_logic;
          Sigma_1     : in  std_logic_vector (NBITS-1 downto 0);
          Sigma_2     : in  std_logic_vector (NBITS-1 downto 0);
          select_sig  : in  std_logic;
          Ia_out      : out std_logic_vector (NBITS-1 downto 0);
          Sigma_out   : out std_logic_vector (NBITS-1 downto 0);
          
          output      : out std_logic_vector (8-1 downto 0)
          );
end entity signal_pipe;

---------------------------------------
-- Behavioral Architecture.
---------------------------------------
architecture behavior of signal_pipe is

-- Defining the components
component sigma is
    generic(NBITS : natural :=18);
    port(
       A       :   in std_logic_vector(NBITS-1 downto 0);
       B       :   in std_logic_vector(NBITS-1 downto 0);        
       use_last:   in std_logic;
       f       :   in std_logic_vector(NBITS-1 downto 0);
       Ia      :   in std_logic_vector(NBITS-1 downto 0);
       m       :   in std_logic_vector(12 downto 0);
       clk     :   in std_logic;
       output  :   out std_logic_vector(NBITS-1 downto 0));
end component sigma;

component comp is
    generic(NBITS : natural :=18);
    port(
        A           :   in std_logic_vector(NBITS-1 downto 0);
        B           :   in std_logic_vector(NBITS-1 downto 0);
        sel_input   :   in std_logic; 
        ----
        input       :   in std_logic_vector(NBITS-1 downto 0);
        sigma       :   in std_logic_vector(NBITS-1 downto 0);
        clk         :   in std_logic;
        output      :   out std_logic_vector(7 downto 0)
        );
end component comp;

component Ia_add is
  generic(NBITS  : natural := 18         -- # bits
          );
    port (A       : in  std_logic_vector (NBITS-1 downto 0);
    			 B       : in  std_logic_vector (NBITS-1 downto 0);
          c       : in  std_logic_vector (2 downto 0);
          output  : out std_logic_vector (NBITS-1 downto 0)
          );
end component Ia_add;
    
    signal temp       : std_logic_vector(NBITS-1 downto 0);
    
    signal Ia_local       : std_logic_vector(NBITS-1 downto 0);
    signal Ia_total       : std_logic_vector(NBITS-1 downto 0);
    signal Ia_total_clk   : std_logic_vector(NBITS-1 downto 0);
    signal Sig_out        : std_logic_vector(NBITS-1 downto 0);
    signal Sig_out_clk    : std_logic_vector(NBITS-1 downto 0);
    signal Input_RGB_1    : std_logic_vector(NBITS-1 downto 0);
    signal Input_RGB_2    : std_logic_vector(NBITS-1 downto 0);
    signal Input_RGB_3    : std_logic_vector(NBITS-1 downto 0);
    signal Ia_loc_in_1    : std_logic_vector(NBITS-1 downto 0);
    signal Ia_loc_in_2    : std_logic_vector(NBITS-1 downto 0);
    
begin  -- behavior
    
-- Initializing the components    
    Ia_loc      :   Ia_add   port map(Ia_loc_in_1, Ia_loc_in_2, c, Ia_local);
    Ia_glob     :   Ia_add   port map(Ia_local, Ia_global, a, Ia_total);
    sigm        :   sigma    port map(Sigma_1, Sigma_2, select_sig, f, Ia_total_clk, m, clk, Sig_out);
    kompresjon  :   comp     port map(Div_1, Div_2, select_div, Input_RGB_3, Sig_out_clk, clk, output);
    
    Sigma_out <= Sig_out;    
    Ia_out    <= Ia_local;

-- Selects which signals to be fed into the Ia_loc module
Select_Ia_process: process(select_Ia, Input_RGB, Input_Lum, Ia_in_1, Ia_in_2)
begin
    if(select_Ia='1') then
        Ia_loc_in_1   <=   Ia_in_1;
        Ia_loc_in_2   <=   Ia_in_2;
    else
        Ia_loc_in_1   <=   Input_RGB;
        Ia_loc_in_2   <=   Input_Lum;
    end if;
end process;

-- Pipelining signals between the submodules                               
pipeline: process(clk)  
begin
    if(rising_edge(clk)) then
        Ia_total_clk  <= Ia_total;
        Sig_out_clk   <= Sig_out;
        Input_RGB_1   <= Input_RGB;
        Input_RGB_2   <= Input_RGB_1;
        Input_RGB_3   <= Input_RGB_2;
    end if;
end process pipeline;

end architecture behavior;  -- of Ia