------------------------------------- 
-- This code is originally from Deschamps06, and the only correction
-- made is the pipelining
-- This module is not generic, but may easely be converted into an generic module
-- by making generic inputs instead of the package.
-------------------------------------
-- Define datapath width
-------------------------------------
package mypackage_1 is
	constant NBITS   :INTEGER := 18;
	constant PBITS   :INTEGER := 8;
	constant PBITS_H :integer := 4;
end mypackage_1;
---------------------------------------------------------------
-- RESTORING Division Algorithm for integer operands with fractional result
-- divisor Y should be greater than dividend X
-- NBITS operands, PBITS quotient and NBITS remainder
---------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.mypackage_1.all;

entity div_rest_frac is
  port (
       X   : in STD_LOGIC_VECTOR (NBITS-1 downto 0);
       Y   : in STD_LOGIC_VECTOR (NBITS-1 downto 0);
       clk : in std_logic;
       Q   : out STD_LOGIC_VECTOR (PBITS-1 downto 0);
       R   : out STD_LOGIC_VECTOR (NBITS-1 downto 0)
     );
end div_rest_frac;

architecture div_arch of div_rest_frac is

component restoring_cell
    port (
        a: in STD_LOGIC_VECTOR (NBITS downto 0);
        b: in STD_LOGIC_VECTOR (NBITS-1 downto 0);
        q: out STD_LOGIC;
        r: out STD_LOGIC_VECTOR (NBITS downto 0)
     );
end component;

type conections is array (0 to PBITS-1) of STD_LOGIC_VECTOR (NBITS downto 0);
Signal wires_in, wires_out: conections;

signal Y_pipe     : std_logic_vector(NBITS-1 downto 0);
signal Q_pipe     : std_logic_vector(PBITS_H-1 downto 0);
signal wires_pipe : std_logic_vector(NBITS downto 0);
signal wires_temp : std_logic_vector(NBITS downto 0);

begin

 wires_in(0) <= X & '0'; 	 

 divisor_1: for i in 0 to PBITS_H-1 generate
       rest_cell: restoring_cell 
       port map (
             a => wires_in(i),
        	 		 b => Y, 
  		       q => Q_pipe(PBITS_H-I-1),
  			    r => wires_out(i));
  end generate;

 divisor_2: for i in PBITS_H to PBITS-1 generate

       rest_cell_2: restoring_cell 
       port map (
             a => wires_in(i),
        	 		 b => Y_pipe, 
  		       q => Q(PBITS-I-1),
  			    r => wires_out(i));

  end generate;

           wires_in(PBITS_H) <= wires_pipe;
           wires_temp   <= wires_out(PBITS_H-1)(NBITS-1 downto 0) & '0';			 

 wires_conections_1: for i in 0 to PBITS_H-2 generate
           wires_in(i+1) <= wires_out(i)(NBITS-1 downto 0) & '0';			 
 end generate;

 wires_conections_2: for i in PBITS_H to PBITS-2 generate
           wires_in(i+1) <= wires_out(i)(NBITS-1 downto 0) & '0';			 
 end generate;


  R <= wires_out(PBITS-1)(NBITS-1 downto 0);

pipelining: process(clk)
begin
    if(rising_edge(clk)) then
        Y_pipe <= Y;
        wires_pipe <= wires_temp;
        Q(PBITS-1 downto (PBITS/2)) <= Q_pipe((PBITS/2)-1 downto 0);
    end if;
end process pipelining;

end div_arch;

---------------------------------------------------------
-- restoring cell
-- division step for restoring base-2 division algorithm
---------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.mypackage_1.all;

entity restoring_cell is
    port (
        a: in STD_LOGIC_VECTOR (NBITS downto 0);
        b: in STD_LOGIC_VECTOR (NBITS-1 downto 0);
  		  q: out STD_LOGIC;
		  r: out STD_LOGIC_VECTOR (NBITS downto 0)
     );
end restoring_cell;

architecture cel_arch of restoring_cell is

signal subst: STD_LOGIC_VECTOR (NBITS downto 0);

begin

subst <= a - b;

multiplexer: process (a,b,subst)
begin
	  if subst(NBITS) = '1' then
	 	 r <= a;
	  else
		 r <= subst;
	  end if; 
end process;

q <= not subst(NBITS);

end cel_arch;


-------------------------------------------------------------------------------
---- VHDL Test Bench Restoring divisor
----
---- Exhaustive Analysis
---- Notes: 		
----
-------------------------------------------------------------------------------
--LIBRARY  IEEE;
--USE IEEE.std_logic_1164.all;
--USE IEEE.std_logic_arith.all;
--USE IEEE.std_logic_unsigned.all;
--USE work.mypackage_1.all;
--
--library IEEE;
--use IEEE.std_logic_1164.all;
--use IEEE.std_logic_arith.all;
--use IEEE.std_logic_unsigned.all;
--use IEEE.math_real.all;
--USE IEEE.STD_LOGIC_TEXTIO.ALL;
--USE STD.TEXTIO.ALL;
--
--ENTITY testbench IS
--END testbench;
--
--ARCHITECTURE behavior OF testbench IS 
--FILE RESULTS: TEXT OPEN WRITE_MODE IS "results.txt";
--
--   COMPONENT div_rest_frac
--	PORT(
--        X    : in STD_LOGIC_VECTOR (NBITS-1 downto 0);
--        Y    : in STD_LOGIC_VECTOR (NBITS-1 downto 0);
--        clk  : in std_logic;
--        Q    : out STD_LOGIC_VECTOR (PBITS-1 downto 0);
--        R    : out STD_LOGIC_VECTOR (NBITS-1 downto 0)
--		);
--    END COMPONENT;
--
--   CONSTANT DELAY: time := 100 ns;
--   CONSTANT print_ok: boolean := false;
--   SIGNAL A         : STD_LOGIC_VECTOR (NBITS-1 downto 0) := (others => '0');
--   SIGNAL B         : STD_LOGIC_VECTOR (NBITS-1 downto 0) := (others => '0');
--   SIGNAL A_delayed : STD_LOGIC_VECTOR (NBITS-1 downto 0) := (others => '0');
--   SIGNAL B_delayed : STD_LOGIC_VECTOR (NBITS-1 downto 0) := (others => '0');   
--   SIGNAL Q         : STD_LOGIC_VECTOR (PBITS-1 downto 0);
--   SIGNAL R         : STD_LOGIC_VECTOR (NBITS-1 downto 0);
--   
--   signal clk : std_logic := '0';
--
--BEGIN
--
--clocking: process
--begin
--    clk <= not clk;
--    wait for 25 ns;
--end process clocking;
--
--delayed_input: process(clk)
--begin
--    if(rising_edge(clk)) then
--        A_delayed <= A;
--        B_delayed <= B;
--    end if;
--end process delayed_input;
--
--   uut: div_rest_frac PORT MAP( X => A, Y => B, clk => clk, Q => Q, R => R );
--
--   tb_gen : PROCESS
--   BEGIN
--       wait for 60 ns;
--		for I in 0 to 2**NBITS-1 loop
--			for J in I+1 to (I*512) loop --2**NBITS-1 loop
--				A <= CONV_STD_LOGIC_VECTOR (I, NBITS);
--				B <= CONV_STD_LOGIC_VECTOR (J, NBITS);
--				WAIT FOR DELAY;
--     		end loop;
--   	end loop;
--   ASSERT (FALSE) SEVERITY ERROR;
--		WAIT FOR DELAY;
--      --wait; -- will wait forever
--   END PROCESS;
--
--check_ans: process(A, B, Q, clk)
--   variable temp             : real;
--   variable real_Q           : real;
--   variable temp_floor       : real;   
--   variable error_value      : real;
--   variable error_too_high   : boolean;
--   variable error_too_high_2 : boolean;   
--   variable worst_error      : real := real(0) ;
--   variable error_percent    : real;
--begin
--      if (NOT(conv_integer(B_delayed) = 0)) then
--          temp          := (real(conv_integer(A_delayed))/real(conv_integer(B_delayed)));              
--          real_Q        := (real(conv_integer(Q))/real(256));
--          error_value   := temp - real_Q;
--          error_percent := error_value / temp;          
--      else
--          temp          := real(0);
--          real_Q        := real(0);
--          error_value   := real(0);
--          error_percent := real(0);
--      end if;
--    if( abs(error_percent) > real(1)) then
----            assert(false) severity error;
--          error_too_high := true;
--          if(falling_edge(clk)) then
--             error_too_high_2 := true;
--          else
--             error_too_high_2 := false;
--          end if;
--    else
--          error_too_high   := false;
--          error_too_high_2 := false;
--    end if;
--end process check_ans;


--   tb_test : PROCESS
--	VARIABLE TX_LOC : LINE;
--	VARIABLE TX_STR : String(1 to 4096);
--	Variable iQ, iR: natural;
--   BEGIN
--		WAIT FOR 10 ns;
--		for I in 0 to 2**NBITS -1 loop
--			for J in I+1 to 2**NBITS -1 loop
--				iQ := CONV_INTEGER(Q);
--				iR := CONV_INTEGER(R);
--				IF (I*2**PBITS /= (J * iQ) + iR) THEN 
--					write(TX_LOC,string'("Error!!! -> A=")); write(TX_LOC, A);
--					write(TX_LOC,string'(" B=")); write(TX_LOC, B);
--					write(TX_LOC,string'(" Q=")); write(TX_LOC, Q);
--					write(TX_LOC,string'(" R=")); write(TX_LOC, R);
--					write(TX_LOC,string'(" (i=")); write(TX_LOC, i);
--					write(TX_LOC,string'(" j=")); write(TX_LOC, j);
--					write(TX_LOC,string'(" iQ=")); write(TX_LOC, iQ);
--					write(TX_LOC,string'(" iR=")); write(TX_LOC, iR);
--					write(TX_LOC, string'(") "));
--					TX_STR(TX_LOC.all'range) := TX_LOC.all;
--					writeline(results, TX_LOC);
--					Deallocate(TX_LOC);
--					ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
--				elsif (iR > J) then
--					write(TX_LOC,string'("Error Remainder bigger than Divider !!! -> A=")); write(TX_LOC, A);
--					write(TX_LOC,string'(" B=")); write(TX_LOC, B);
--					write(TX_LOC,string'(" Q=")); write(TX_LOC, Q);
--					write(TX_LOC,string'(" R=")); write(TX_LOC, R);
--					write(TX_LOC,string'(" (i=")); write(TX_LOC, i);
--					write(TX_LOC,string'(" j=")); write(TX_LOC, j);
--					write(TX_LOC,string'(" iQ=")); write(TX_LOC, iQ);
--					write(TX_LOC,string'(" iR=")); write(TX_LOC, iR);
--					write(TX_LOC, string'(") "));
--					TX_STR(TX_LOC.all'range) := TX_LOC.all;
--					writeline(results, TX_LOC);
--					Deallocate(TX_LOC);
--					ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
--
--			   elsif (print_ok) then
--					write(TX_LOC,string'("OK -> A=")); write(TX_LOC, A);
--					write(TX_LOC,string'(" B=")); write(TX_LOC, B);
--					write(TX_LOC,string'(" Q=")); write(TX_LOC, Q);
--					write(TX_LOC,string'(" R=")); write(TX_LOC, R);
--					write(TX_LOC, string'(" "));
--					TX_STR(TX_LOC.all'range) := TX_LOC.all;
--					writeline(results, TX_LOC);
--					Deallocate(TX_LOC);
--					ASSERT (FALSE) REPORT TX_STR SEVERITY WARNING;
--				END IF;
--				WAIT FOR DELAY;
--    		end loop;
--   	end loop;
--		ASSERT (FALSE) REPORT
--		"Simulation successful (not a failure).  No problems detected. "
--		SEVERITY FAILURE;
--   END PROCESS;

--END architecture;