---------------------------------------------------------------
-- Adder/Subtractor from Deschamps06
-- with minor adjustments to fit this project
-- This module is fully generic
---------------------------------------------------------------

library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

---------------------------------------------------------------
-- Defining the entity
---------------------------------------------------------------

entity addsub is
generic(NBITS : natural := 18);
port 
   (x       : in std_logic_vector(NBITS-1 downto 0);
    y       : in std_logic_vector(NBITS-1 downto 0);
    control : in std_logic;
    d_in    : in std_logic;
    z       : out std_logic_vector(NBITS-1 downto 0);
    d_out   : out std_logic
   );
end addsub;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture circuit of addsub is
   signal minus_y   : std_logic_vector(NBITS-1 downto 0);
   signal operand_2 : std_logic_vector(NBITS-1 downto 0);
   signal carries   : std_logic_vector(NBITS downto 0);

begin
    
-- Inverts signal if sub and not add

   invert: for i in 0 to NBITS-1 generate minus_y(i) <= NOT y(i); end generate;
      with control select operand_2 <= y when '0', minus_y when others;
      
-- Generates the chain

   carries(0) <= control xor d_in;
   adder: for i in 0 to NBITS-1 generate
      iterative_step:
      z(i) <= (x(i) xor operand_2(i) xor carries(i));
      carries(i+1) <= (x(i) and operand_2(i)) or (x(i) and carries(i)) or (operand_2(i) and carries(i));
   end generate; 
   
-- Carry-out signal

   d_out <= carries(NBITS) xor control;

end circuit;
