---------------------------------------
-- This module computes the adaption level of the intensity.
-- Both Ia_local, Ia_global and Ia_total use this hardwaremodule.
-- output = c*A + (1-c)*B
---------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

---------------------------------------
-- Defining the Entity
---------------------------------------
entity Ia_add is
  generic(NBITS  : natural := 18         -- # bits
          );
    port (A       : in  std_logic_vector (NBITS-1 downto 0);
    			 B       : in  std_logic_vector (NBITS-1 downto 0);
          c       : in  std_logic_vector (2 downto 0);
          output  : out std_logic_vector (NBITS-1 downto 0)
          );
end entity Ia_add;

---------------------------------------
-- Behavioral Architecture
---------------------------------------
architecture behavior of Ia_add is
    
-- Defining the components
component add_g is
  generic(left : natural := NBITS+2         -- top bit
          );
    port (a    : in  std_logic_vector (left downto 0);
          b    : in  std_logic_vector (left downto 0);
          cin  : in  std_logic;
          sum  : out std_logic_vector (left downto 0);
          cout : out std_logic);
end component add_g;    
    
    signal temp      : std_logic_vector(NBITS+2 downto 0);
    
    signal shifted_1     : std_logic_vector(NBITS+2 downto 0);
    signal shifted_2     : std_logic_vector(NBITS+2 downto 0);
    signal shifted_3     : std_logic_vector(NBITS+2 downto 0);
    signal shifted_4     : std_logic_vector(NBITS+2 downto 0);
    signal partial_sum_1 : std_logic_vector(NBITS+2 downto 0);
    signal partial_sum_2 : std_logic_vector(NBITS+2 downto 0);
    
    signal cout_1        : std_logic;
    signal cout_2        : std_logic;
    signal cout_3        : std_logic;
        
begin  -- behavior

-- Initializing the components    
    adder_1   :   add_g   port map(shifted_1, shifted_2, '0', partial_sum_1, cout_1);
    adder_2   :   add_g   port map(shifted_3, shifted_4, '0', partial_sum_2, cout_2);
    adder_3   :   add_g   port map(partial_sum_1, partial_sum_2, '0', temp, cout_3);

-- Adjusting the adaption-level according to the c-input    
process (A, B, c, temp)    
begin
   case c(2 downto 0) is
       when "000" => shifted_1 <= (B(NBITS-1 downto 0) & "000");
                     shifted_2 <= (others=>'0');
                     shifted_3 <= (others=>'0');
                     shifted_4 <= (others=>'0');
       when "001" => shifted_1 <= ("0" & B(NBITS-1 downto 0) & "00");
                     shifted_2 <= ("00" & B(NBITS-1 downto 0) & "0");
                     shifted_3 <= ("000" & A(NBITS-1 downto 0));
                     shifted_4 <= ("000" & B(NBITS-1 downto 0));
       when "010" => shifted_1 <= ("0" & B(NBITS-1 downto 0) & "00");
                     shifted_2 <= ("000" & B(NBITS-1 downto 0));
                     shifted_3 <= ("00" & A(NBITS-1 downto 0) & "0");
                     shifted_4 <= ("000" & B(NBITS-1 downto 0));
       when "011" => shifted_1 <= ("0" & B(NBITS-1 downto 0) & "00");
                     shifted_2 <= ("00" & A(NBITS-1 downto 0) & "0");
                     shifted_3 <= ("000" & A(NBITS-1 downto 0));
                     shifted_4 <= ("000" & B(NBITS-1 downto 0));
       when "100" => shifted_1 <= ("000" & B(NBITS-1 downto 0));
                     shifted_2 <= ("00" & B(NBITS-1 downto 0) & "0");
                     shifted_3 <= ("0" & A(NBITS-1 downto 0) & "00");
                     shifted_4 <= ("000" & B(NBITS-1 downto 0));
       when "101" => shifted_1 <= ("00" & B(NBITS-1 downto 0) & "0");
                     shifted_2 <= ("0" & A(NBITS-1 downto 0) & "00");
                     shifted_3 <= ("000" & A(NBITS-1 downto 0));
                     shifted_4 <= ("000" & B(NBITS-1 downto 0));
       when "110" => shifted_1 <= ("000" & B(NBITS-1 downto 0));
                     shifted_2 <= ("00" & A(NBITS-1 downto 0) & "0");
                     shifted_3 <= ("0" & A(NBITS-1 downto 0) & "00");
                     shifted_4 <= ("000" & B(NBITS-1 downto 0));
       when "111" => shifted_1 <= (others=>'0');
                     shifted_2 <= (A(NBITS-1 downto 0) & "000");
                     shifted_3 <= (others=>'0');
                     shifted_4 <= (others=>'0');
       when others =>shifted_1 <= (others=>'0');
                     shifted_2 <= (others=>'0');
                     shifted_3 <= (others=>'0');
                     shifted_4 <= (others=>'0');
   end case;
------output = c*A + (1-c)*B; 
   output(NBITS-1 downto 0) <= temp(NBITS+2 downto 3);
end process;
end architecture behavior;  -- of Ia