---------------------------------------------------------------
-- Logarithm unit designed after the documentation in Abed03
-- with the 3-region error correction and
-- with minor adjustments to fit this project
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

---------------------------------------------------------------
-- Defining the entity
---------------------------------------------------------------

entity Logarithm_Abed03_3region is
    port(
       input   :   in std_logic_vector(17 downto 0);
       zero_in :   out std_logic;
       output  :   out std_logic_vector(17 downto 0));
end entity Logarithm_Abed03_3region;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavioral of Logarithm_Abed03_3region is
    
-- Defining the components    
    
 component leading_one_abed00   
    port(    
    input     : in std_logic_vector(17 downto 0);
    zero_in   : out std_logic;
    output    : out std_logic_vector(17 downto 0));
  end component leading_one_abed00;
  
  component ROM18x5
      port(
      adress  : in std_logic_vector(17 downto 0);
      value   : out std_logic_vector(4 downto 0)  );
  end component ROM18x5;
   
  component Shifter18 
      port(
      number_of_shifts   : in std_logic_vector(4 downto 0);
      input              : in std_logic_vector(17 downto 0);
      output             : out std_logic_vector(17 downto 0));
  end component Shifter18;
  
  component abed03_3region_error_correction 
      port(
      input   : in std_logic_vector(17 downto 0);
      output  : out std_logic_vector(17 downto 0));
  end component abed03_3region_error_correction;  

	signal internal    : std_logic_vector(17 downto 0);    
	signal shifted     : std_logic_vector(17 downto 0);    	
	signal output_temp : std_logic_vector(17 downto 0);    		
	signal LOD18       : std_logic_vector(17 downto 0);    
	signal LOD5        : std_logic_vector(4 downto 0);    
begin

-- Initializing the components
LOD_circuit: leading_one_abed00 				         port map(internal, zero_in, LOD18);
ROM:         ROM18x5                				     port map(LOD18, LOD5);
Shifter:     Shifter18                  				 port map(LOD5, internal, shifted);
Abed03_3reg: abed03_3region_error_correction port map(output_temp, output);

-- connectiong the internal signals
no_clocking:process(input, LOD5, shifted)
begin
           internal(17 downto 0)      <= input(17 downto 0);
           output_temp(17 downto 13)  <= LOD5(4 downto 0);
           output_temp(12 downto 0)   <= shifted(17 downto 5);
end process no_clocking;

end architecture;
