---------------------------------------
-- This module is a multiplexer that decides which color to
-- feed the signal path with in the samll architecture.
-- This module is fully generic
---------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

---------------------------------------
-- Defining the entity
---------------------------------------
entity RGB_mux is
  generic(NBITS : natural := 18
          );
    port (R          : in  std_logic_vector (NBITS-1 downto 0);
          G          : in  std_logic_vector (NBITS-1 downto 0);
          B          : in  std_logic_vector (NBITS-1 downto 0);
          Ia_g_R     : in  std_logic_vector (NBITS-1 downto 0);
          Ia_g_G     : in  std_logic_vector (NBITS-1 downto 0);
          Ia_g_B     : in  std_logic_vector (NBITS-1 downto 0);
          cntrl_RGB  : in  std_logic_vector (1 downto 0);
          clk        : in  std_logic;
          RGB_out    : out std_logic_vector (NBITS-1 downto 0);
          Ia_out     : out std_logic_vector (NBITS-1 downto 0)
          );
end entity RGB_mux;

---------------------------------------
-- Behavioral Architecture.
---------------------------------------
architecture behavior of RGB_mux is

begin  -- behavior

-- Multiplexing with the use of a case.
mux: process(R, G, B, Ia_g_R, Ia_g_G, Ia_g_B, cntrl_RGB)
begin
     case cntrl_RGB is
         when "01" =>   RGB_out <= R;
                        Ia_out  <= Ia_g_R;
         when "10" =>   RGB_out <= G;
                        Ia_out  <= Ia_g_G;
         when "11" =>   RGB_out <= B;
                        Ia_out  <= Ia_g_B;
         when others => RGB_out <= (others=>'0');
                        Ia_out  <= (others=>'0');
     end case;
end process mux; 

end architecture behavior;  -- of add_g   