---------------------------------------------------------------
-- This module estimates the average of 307200 N-bit samples.
-- Since the correct number of samples is 307200, this module is 
-- designed for VGA-images
-- The N-bit input is generic
---------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;

---------------------------------------------------------------
-- Defining the Entity
---------------------------------------------------------------

entity avg is
    generic(NBITS : natural :=18);
    port(
       input   :   in std_logic_vector(NBITS-1 downto 0);
       enable  :   in std_logic;
       reset   :   in std_logic;
       clk     :   in std_logic;
       output  :   out std_logic_vector(NBITS-1 downto 0));
end entity avg;

---------------------------------------------------------------
-- Behavioral Architecture of the average-module
---------------------------------------------------------------

architecture behavioral of avg is

-- Defining the adder-component

component add_g is
  generic(left : natural         -- top bit
          );
    port (a    : in  std_logic_vector (left downto 0);
          b    : in  std_logic_vector (left downto 0);
          cin  : in  std_logic;
          sum  : out std_logic_vector (left downto 0);
          cout : out std_logic);
end component;


	signal add_1             : std_logic_vector(NBITS-1+10 downto 0);    
	signal add_2             : std_logic_vector(NBITS-1+9 downto 0);    	
	signal temp_1            : std_logic_vector(NBITS-1+10 downto 0);    
	signal temp_2            : std_logic_vector(NBITS-1+9 downto 0);    	
	signal input_big         : std_logic_vector(NBITS-1+10 downto 0); 
	signal out_sub_circuit_1 : std_logic_vector(NBITS-1+10 downto 0);  	
	signal out_sub_circuit_2 : std_logic_vector(NBITS-1+10 downto 0); 
	signal out_sub_circuit_3 : std_logic_vector(NBITS-1+10 downto 0); 
	signal out_sub_circuit_4 : std_logic_vector(NBITS-1+10 downto 0); 	
   signal in_1              : std_logic_vector(NBITS-1+10 downto 0); 				
   signal in_2              : std_logic_vector(NBITS-1+10 downto 0); 				
   signal in_3              : std_logic_vector(NBITS-1+10 downto 0); 				
   signal in_4              : std_logic_vector(NBITS-1+10 downto 0); 
   signal in_5              : std_logic_vector(NBITS-1+10 downto 0); 								
	signal step_1_big        : std_logic_vector(NBITS-1+9 downto 0);    	
	signal step_1            : std_logic_vector(NBITS-1 downto 0);    	
	signal step_2            : std_logic_vector(NBITS-1 downto 0);  
	
	signal counter           : std_logic_vector(9 downto 0);
	signal internal_clk      : std_logic;	
	 
	signal cout_1      : std_logic;
	signal cout_2      : std_logic;
	signal cout_3      : std_logic;
	signal cout_4      : std_logic;
	signal cout_5      : std_logic;
	signal cout_6      : std_logic;
	
	signal internal    : std_logic_vector(NBITS-1 downto 0);    	
	signal overflow    : std_logic;
	   
begin

-- Initializing the adders
   
   adder_1           : add_g   generic map(NBITS-1+10)  port map(temp_1, input_big, '0', add_1, cout_1);
   adder_2           : add_g   generic map(NBITS-1+9)   port map(temp_2, step_1_big, add_1(9), add_2, cout_2);
   
   sub_circuit_1     : add_g   generic map(NBITS-1+10)   port map(in_1, in_2, '0', out_sub_circuit_1, cout_3);
   sub_circuit_2     : add_g   generic map(NBITS-1+10)   port map(in_3, in_4, '0', out_sub_circuit_2, cout_4);
   sub_circuit_3     : add_g   generic map(NBITS-1+10)   port map(in_5, out_sub_circuit_1, '0', out_sub_circuit_3, cout_5);
   sub_circuit_4     : add_g   generic map(NBITS-1+10)   port map(out_sub_circuit_2, out_sub_circuit_3, '0', out_sub_circuit_4, cout_6);
   
-- Assigning the internal signals   

   input_big    <= ("0000000000" & input);
   step_1_big   <= ("000000000" & step_1);
   step_1       <= add_1(NBITS-1+10 downto 10);
   output       <= out_sub_circuit_4(NBITS-1+9 downto 9); 
   in_1         <= ("0" & temp_2);
   in_2         <= ("00" & temp_2(NBITS-1+9 downto 1));
   in_3         <= ("0000" & temp_2(NBITS-1+9 downto 3));
   in_4         <= ("00000" & temp_2(NBITS-1+9 downto 4));
   in_5         <= ("0000000" & temp_2(NBITS-1+9 downto 6));
   
   overflow     <=  out_sub_circuit_4(NBITS-1+10);
   internal     <=  out_sub_circuit_4(NBITS-1+10 downto 10);
   
-- generating the internal clk for the secound adder-step.

   internal_clk <= counter(0) AND counter(1) AND counter(2) AND counter(3) AND counter(4) AND 
                   counter(5) AND counter(6) AND counter(7) AND counter(8) AND counter(9);

-- The first adder-step
       
process(clk, reset)
begin    
    if(reset = '1' or internal_clk='1') then
       temp_1          <= (others => '0');        
    else
       if(rising_edge(clk)) then 
          if(enable = '1') then     
             temp_1 <= add_1;
          end if;
       end if;  
    end if;
end process;

-- The second adder-step

process(internal_clk, reset)
begin  
    if(reset = '1') then
       temp_2 <= (others => '0');
    else
       if(rising_edge(internal_clk)) then       
          temp_2 <= add_2;
       end if;  
    end if;
end process;

-- Keeps track of the number of samples.
-- The output is valid when the number of samples is 307200

process(clk, reset)
begin
    if(reset='1') then
       counter <= (others => '0');
    else
       if(rising_edge(clk)) then
           if(enable = '1') then
               counter <= counter+1;
           end if;
       end if;
    end if;
end process;        

end architecture;