---------------------------------------
-- This module is the compression unit that compresses the pixelvalues
-- in the final step in the signal path.
-- This module is fully generic
---------------------------------------
---- Define datapath width
---------------------------------------
package mypackage is
	constant NBITS   :INTEGER := 18;
	constant PBITS   :INTEGER := 8;
	constant PBITS_H :integer := 4;
end mypackage;

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;
use work.mypackage.all;

---------------------------------------
-- Defining the entity
---------------------------------------

entity comp is
    generic(NBITS : natural :=18);
    port(
        A           :   in std_logic_vector(NBITS-1 downto 0);
        B           :   in std_logic_vector(NBITS-1 downto 0);
        sel_input   :   in std_logic; 
        ----
        input       :   in std_logic_vector(NBITS-1 downto 0);
        sigma       :   in std_logic_vector(NBITS-1 downto 0);
        clk         :   in std_logic;
        output      :   out std_logic_vector(7 downto 0)
        );
end entity comp;

---------------------------------------
-- Behavioral Architecture
---------------------------------------

architecture behavioral of comp is

-- Defining the components
    
component add_g is
  generic(left : natural := 17         -- top bit
          );
    port (a    : in  std_logic_vector (left downto 0);
          b    : in  std_logic_vector (left downto 0);
          cin  : in  std_logic;
          sum  : out std_logic_vector (left downto 0);
          cout : out std_logic);
end component;

component div_rest_frac is
  port (
       X   : in STD_LOGIC_VECTOR (NBITS-1 downto 0);
       Y   : in STD_LOGIC_VECTOR (NBITS-1 downto 0);
       clk : in std_logic;
       Q   : out STD_LOGIC_VECTOR (PBITS-1 downto 0);
       R   : out STD_LOGIC_VECTOR (NBITS-1 downto 0)
     );
end component;


	signal Reg_1       : std_logic_vector(NBITS-1 downto 0);    
	signal Reg_2       : std_logic_vector(NBITS-1 downto 0);    	
	signal divisor_1   : std_logic_vector(NBITS-1 downto 0);
	signal div_1       : std_logic_vector(NBITS-1 downto 0);	
	signal R           : std_logic_vector(NBITS-1 downto 0);
	
	signal cout        : std_logic;
	   
begin

-- Initializing the components
   
   Adder   :   add_g         port map(input, sigma, '0', divisor_1, cout);
   Div     :   div_rest_frac port map(Reg_1, Reg_2, clk, output, R);

-- Overflow-detection   
process(divisor_1)
begin
    if(cout = '1') then
        div_1 <= (others => '1');
    else
        div_1 <= divisor_1;
    end if;
end process;

-- Selecting which signals to divide
-- If the sel_input signal is high, then the output equals A/B
process(clk)
begin
    if(rising_edge(clk)) then
        if(sel_input = '1') then
            Reg_1 <= A;
            Reg_2 <= B;
        else
            Reg_1 <= input;
            Reg_2 <= div_1;
        end if;
    end if;
end process;        

end architecture;