---------------------------------------
-- This module computes the initial value of f.
-- Output = 1.375*input + ((-1)^cntrl_f)*f_user
-- This module is fully generic
---------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;

---------------------------------------
-- Entity
---------------------------------------
entity f_inn is
	 generic( NBITS : natural := 18
	 );
    port(       
       input   :   in std_logic_vector(NBITS-1 downto 0);
       f_user  :   in std_logic_vector(NBITS-1 downto 0);
       cntrl_f :   in   std_logic;
       output  :   out std_logic_vector(NBITS-1 downto 0)
    );
end entity f_inn;

---------------------------------------
-- Behavioral Architecture
---------------------------------------
architecture behavioral of f_inn is

-- Defining the components
component add_g is
  generic(left : natural := NBITS+3
          );
    port (a    : in  std_logic_vector (left downto 0);
          b    : in  std_logic_vector (left downto 0);
          cin  : in  std_logic;
          sum  : out std_logic_vector (left downto 0);
          cout : out std_logic);
end component add_g;

component addsub
   port 
      (x, y: in std_logic_vector(NBITS-1 downto 0);
      control, d_in: in std_logic;
      z: out std_logic_vector(NBITS-1 downto 0);
      d_out: out std_logic
   );
end component;

	signal temp          : std_logic_vector(NBITS+3 downto 0);    
	
	signal shifted_1     : std_logic_vector(NBITS+3 downto 0);    
	signal shifted_2     : std_logic_vector(NBITS+3 downto 0);    
	signal shifted_3     : std_logic_vector(NBITS+3 downto 0);    
	signal partial_sum_1 : std_logic_vector(NBITS+3 downto 0);    
	signal partial_sum_2 : std_logic_vector(NBITS+3 downto 0);    
	signal automatic_f   : std_logic_vector(NBITS-1 downto 0);    	
	signal f_intern      : std_logic_vector(NBITS downto 0);    	
	
	signal cout_1        : std_logic;
	signal cout_2        : std_logic;
	
begin

-- Initializing the components

   Adder_1   :   add_g   port map(shifted_1, shifted_2, '0', partial_sum_1, cout_1);
   Adder_2   :   add_g   port map(shifted_3, partial_sum_1, '0', partial_sum_2, cout_2);
   
   add_sub   :   addsub  port map(automatic_f, f_user, cntrl_f, '0', f_intern(NBITS-1 downto 0), f_intern(NBITS));   
   
   shifted_1   <= ("0" & input(NBITS-1 downto 0) & "000");
   shifted_2   <= ("000" & input(NBITS-1 downto 0) & "0");
   shifted_3   <= ("0000" & input(NBITS-1 downto 0));

-- Overflow detection   
process(f_intern)
begin
   if(f_intern(NBITS)='1') then
      output <= (others => '1');
   else
      output <= f_intern(NBITS-1 downto 0);
   end if;   
end process;

-- Overflow detection      
process(partial_sum_2, cout_1, cout_2)
begin
   if(temp(NBITS+3)='1' OR cout_1 = '1' OR cout_2 = '1') then
      automatic_f(NBITS-1 downto 0) <= (others => '1');
   else
      automatic_f(NBITS-1 downto 0) <= partial_sum_2(NBITS+2 downto 3);
   end if;
end process;

end architecture;