---------------------------------------
-- This module calculates the minimum value of a stream of values
-- until reset.
-- This module is fully generic
---------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

---------------------------------------
-- Entity
---------------------------------------
entity minimum is
  generic(NBITS  : natural := 18         -- # bits
          );
    port (input : in  std_logic_vector (NBITS-1 downto 0);
          clk   : in  std_logic;    
          reset : in  std_logic;
          output: out std_logic_vector (NBITS-1 downto 0)
          );
end entity minimum;

---------------------------------------
-- Architecture
---------------------------------------
architecture behavior of minimum is
    signal internal : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal empty    : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
begin  -- behavior
    empty   <= (others=>'0');

-- Checking if the input is a smaller value than the internal register.
-- If so, then overwrite the internal register with the input value.     
process (clk, internal)
   begin
   if(rising_edge(clk)) then
       if(reset='1') then            -- Active high reset
           internal <= (others =>'1');
       else
          if(input < internal AND (input/=empty)) then
              internal <= input;
          end if;
       end if;
   end if;
   output <= internal;                
end process;
end architecture behavior;  -- of minimum