---------------------------------------
-- This module either computes the parameter sigma, or if the use_last signal is high,
-- it computes output = 2^(A*B) 
-- This module is not fully generic.
---------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;

---------------------------------------
-- Defines the Entity
---------------------------------------
entity sigma is
    --- sigma = (f*Ia)^m;  => 2^((((-1)^cntrl)*f_user + f) + log2(Ia) * m)
    generic(NBITS : natural :=18);
    port(
       -- A and B is the inputs to the multiplicator if
       -- use_last == '1'
       A       :   in std_logic_vector(NBITS-1 downto 0);
       B       :   in std_logic_vector(NBITS-1 downto 0);        
       use_last:   in std_logic;
       -- When the system is in the compression-phase, this is the valid inputs
       f       :   in std_logic_vector(NBITS-1 downto 0);
       Ia      :   in std_logic_vector(NBITS-1 downto 0);
       m       :   in std_logic_vector(12 downto 0);
       clk     :   in std_logic;
       output  :   out std_logic_vector(NBITS-1 downto 0));
end entity sigma;

---------------------------------------
-- Behavioral architecture
---------------------------------------
architecture behavioral of sigma is

-- Defining the components
component addsub
   port 
      (x, y: in std_logic_vector(NBITS-1 downto 0);
      control, d_in: in std_logic;
      z: out std_logic_vector(NBITS-1 downto 0);
      d_out: out std_logic
   );
end component;

component add_g is
  generic(left : natural := 17         -- top bit
          );
    port (a    : in  std_logic_vector (left downto 0);
          b    : in  std_logic_vector (left downto 0);
          cin  : in  std_logic;
          sum  : out std_logic_vector (left downto 0);
          cout : out std_logic);
end component;

component Logarithm_Abed03_3region
      port(
      input              : in std_logic_vector(NBITS-1 downto 0);
      zero_in            : out std_logic;
      output             : out std_logic_vector(NBITS-1 downto 0));
end component Logarithm_Abed03_3region;
  
component antilogarithm
      port(
      input              : in std_logic_vector(NBITS-1 downto 0);
      output             : out std_logic_vector(NBITS-1 downto 0));
end component antilogarithm;

component carry_save_mult
   generic(NBITS :natural := NBITS;
	        MBITS :natural := work.sigma.NBITS);
    Port ( X : in std_logic_vector(NBITS-1 downto 0);
           Y : in std_logic_vector(NBITS-1 downto 0);
           P : out std_logic_vector(NBITS+NBITS-1 downto 0)
    );
end component carry_save_mult;
  	
	signal Ia_log      : std_logic_vector(NBITS-1 downto 0);    
	signal Ia_log_2    : std_logic_vector(NBITS downto 0);    	
	signal temp_1      : std_logic_vector(NBITS-1 downto 0);    	
	signal temp_1_ovfl : std_logic_vector(NBITS-1 downto 0);    		
   signal temp_2      : std_logic_vector(NBITS-1 downto 0);    		
	signal m_2         : std_logic_vector(NBITS-1 downto 0);    
	signal mult_1      : std_logic_vector(NBITS-1 downto 0);    
	signal mult_in_1   : std_logic_vector(NBITS-1 downto 0);    
	signal mult_in_2   : std_logic_vector(NBITS-1 downto 0);    
	signal temp_big    : std_logic_vector(NBITS+NBITS-1 downto 0);
	
	signal zero_f      : std_logic;
	signal zero_Ia     : std_logic;
	signal zero_m      : std_logic;
	signal cout        : std_logic;
	signal check_owfl  : std_logic;
	   
begin

-- Initializing the components   
   logarithm_Ia    : Logarithm_Abed03_3region  port map(Ia, zero_Ia, Ia_log);
   adder           : add_g                     port map(f, Ia_log, '0', temp_1, cout);
   multiplier      : carry_save_mult           port map(mult_in_1, mult_in_2, temp_big);
   antilog         : antilogarithm             port map(mult_1, output);   

-- pipeline
process(clk)
begin    
 if(rising_edge(clk)) then   
     temp_2 <= temp_1_ovfl;
     m_2    <= ("00000" & m);
 end if;
end process;

-- choose valid multiplier input
process(temp_2, m_2, A, B, use_last)
begin
    if(use_last = '0') then        
       mult_in_1 <= temp_2;
       mult_in_2 <= m_2;
    else
       mult_in_1 <= A;
       mult_in_2 <= B;   
    end if;        
end process;

-- chooses valid multiplier-output
process(temp_big)
begin
    mult_1(NBITS-1 downto 0) <= temp_big(NBITS+NBITS-1-5 downto NBITS-5); 
end process;    

-- Check for overflow
process(temp_1, cout)
begin
    if(cout='1') then
        temp_1_ovfl(NBITS-1 downto 0) <= (others => '1');
    else
        temp_1_ovfl(NBITS-1 downto 0) <= temp_1(NBITS-1 downto 0);
    end if;
end process;
           
end architecture;