---------------------------------------
-- This module represents the top-level of the small architecture with the 
-- seriell color-prosessing.
-- This module is not fully generic.
---------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;

---------------------------------------
-- Defines the Entity
---------------------------------------
entity top_level_small is
    port(
       clk         : in   std_logic;
       reset_in    : in   std_logic;
       go          : in   std_logic;
       
       R           : in   std_logic_vector (18-1 downto 0);
       G           : in   std_logic_vector (18-1 downto 0);
       B           : in   std_logic_vector (18-1 downto 0);
       
       f_user_in   : in   std_logic_vector (18-1 downto 0);
       cntrl_f     : in   std_logic;
       m_in        : in   std_logic_vector (12 downto 0);
       cntrl_m     : in   std_logic;
       c           : in   std_logic_vector (2 downto 0);
       a           : in   std_logic_vector (2 downto 0);
       
       output_R    : out  std_logic_vector (7 downto 0);
       output_G    : out  std_logic_vector (7 downto 0);
       output_B    : out  std_logic_vector (7 downto 0)
       );
end entity top_level_small;

---------------------------------------
-- Behavioral Architecture
---------------------------------------
architecture behavior of top_level_small is
    
    constant NBITS : natural := 18;

-- Defining the components
component signal_pipe is
  generic(NBITS  : natural := NBITS         -- # bits
          );
    port (Input_RGB   : in  std_logic_vector (NBITS-1 downto 0);
    			 Input_Lum   : in  std_logic_vector (NBITS-1 downto 0);          
          f           : in  std_logic_vector (NBITS-1 downto 0);
          a           : in  std_logic_vector (2 downto 0);
          c           : in  std_logic_vector (2 downto 0);          
          m           : in  std_logic_vector (12 downto 0);          
          Ia_global   : in  std_logic_vector (NBITS-1 downto 0);
          clk         : in  std_logic;
          Ia_in_1     : in  std_logic_vector (NBITS-1 downto 0);
          Ia_in_2     : in  std_logic_vector (NBITS-1 downto 0);          
          Div_1       : in  std_logic_vector (NBITS-1 downto 0);
          Div_2       : in  std_logic_vector (NBITS-1 downto 0);
          select_div  : in  std_logic;
          select_Ia   : in  std_logic;
          Sigma_1     : in  std_logic_vector (NBITS-1 downto 0);
          Sigma_2     : in  std_logic_vector (NBITS-1 downto 0);
          select_sig  : in  std_logic;
          Ia_out      : out std_logic_vector (NBITS-1 downto 0);
          Sigma_out   : out std_logic_vector (NBITS-1 downto 0);
          
          output      : out std_logic_vector (8-1 downto 0)
          );
end component signal_pipe;

component luminance is
  generic(NBITS  : natural := NBITS         -- # bits
          );
    port (red   : in  std_logic_vector (NBITS-1 downto 0);
          green : in  std_logic_vector (NBITS-1 downto 0);
          blue  : in  std_logic_vector (NBITS-1 downto 0);
          output: out std_logic_vector (NBITS-1 downto 0)
          );
end component luminance;

component param is
  generic(NBITS  : natural := NBITS         -- # bits
          );
    port (clk         : in  std_logic;
          reset       : in  std_logic;
          comp_phase  : in  std_logic;
          go          : in  std_logic;
          
          R           : in  std_logic_vector (NBITS-1 downto 0);
          G           : in  std_logic_vector (NBITS-1 downto 0);
          B           : in  std_logic_vector (NBITS-1 downto 0);
          R_enable    : in  std_logic;
          G_enable    : in  std_logic;
          B_enable    : in  std_logic;
    			 Input_Lum   : in  std_logic_vector (NBITS-1 downto 0);  
    			 Lum_enable  : in  std_logic;                  
          m_in        : in  std_logic_vector (12 downto 0);          
          cntrl_m     : in  std_logic; 
          f_user      : in  std_logic_vector (NBITS-1 downto 0);
          cntrl_f     : in  std_logic;          
          
          Div_ans     : in  std_logic_vector (8-1 downto 0);
          Sig_ans     : in  std_logic_vector (NBITS-1 downto 0);
          Ia_in       : in  std_logic_vector (NBITS-1 downto 0);
          select_sig  : out std_logic;
          select_div  : out std_logic; 
          select_Ia   : out std_logic;                         
          Ia_out_1    : out std_logic_vector (NBITS-1 downto 0);
          Ia_out_2    : out std_logic_vector (NBITS-1 downto 0);
          Sigma_1     : out std_logic_vector (NBITS-1 downto 0);
          Sigma_2     : out std_logic_vector (NBITS-1 downto 0);          
          Div_1       : out std_logic_vector (NBITS-1 downto 0);
          Div_2       : out std_logic_vector (NBITS-1 downto 0);
            
          Ia_g_R_out  : out std_logic_vector (NBITS-1 downto 0);
          Ia_g_G_out  : out std_logic_vector (NBITS-1 downto 0);
          Ia_g_B_out  : out std_logic_vector (NBITS-1 downto 0);          
          m_out       : out std_logic_vector (12 downto 0);  
          f           : out std_logic_vector (NBITS-1 downto 0)
          );
end component param;

component control_small is
    port (
          clk         : in  std_logic;
          reset       : in  std_logic;
          go          : in  std_logic;          
          reset_out   : out std_logic;                              
    			 R_enable    : out std_logic;
          G_enable    : out std_logic;
          B_enable    : out std_logic;
    			 Lum_enable  : out std_logic;                  
          comp_phase  : out std_logic;
          cntrl_RGB   : out std_logic_vector(1 downto 0)
          );
end component control_small;

component RGB_mux is
  generic(NBITS : natural := NBITS         -- top bit
          );
    port (R          : in  std_logic_vector (NBITS-1 downto 0);
          G          : in  std_logic_vector (NBITS-1 downto 0);
          B          : in  std_logic_vector (NBITS-1 downto 0);
          Ia_g_R     : in  std_logic_vector (NBITS-1 downto 0);
          Ia_g_G     : in  std_logic_vector (NBITS-1 downto 0);
          Ia_g_B     : in  std_logic_vector (NBITS-1 downto 0);
          cntrl_RGB  : in  std_logic_vector (1 downto 0);
          clk        : in  std_logic;
          RGB_out    : out std_logic_vector (NBITS-1 downto 0);
          Ia_out     : out std_logic_vector (NBITS-1 downto 0)
          );
end component RGB_mux;

-- Defining the internal signals
    signal Luminance_value : std_logic_vector(NBITS-1 downto 0);    
    signal f               : std_logic_vector(NBITS-1 downto 0);
    signal m               : std_logic_vector(12 downto 0);
    signal Ia_g_R          : std_logic_vector(NBITS-1 downto 0);    
    signal Ia_g_G          : std_logic_vector(NBITS-1 downto 0);    
    signal Ia_g_B          : std_logic_vector(NBITS-1 downto 0);   
    signal Div_1           : std_logic_vector(NBITS-1 downto 0);     
    signal Div_2           : std_logic_vector(NBITS-1 downto 0);     
    signal select_div      : std_logic;
    signal Sigma_1         : std_logic_vector(NBITS-1 downto 0);     
    signal Sigma_2         : std_logic_vector(NBITS-1 downto 0);            
    signal Sigma_out       : std_logic_vector(NBITS-1 downto 0);     
    signal Ia_1            : std_logic_vector(NBITS-1 downto 0);
    signal Ia_2            : std_logic_vector(NBITS-1 downto 0);
    signal Ia_out          : std_logic_vector(NBITS-1 downto 0);
    signal select_sig      : std_logic;
    signal R_enable        : std_logic;
    signal G_enable        : std_logic;
    signal B_enable        : std_logic;
    signal Lum_enable      : std_logic;
    signal comp_phase      : std_logic;
    signal reset           : std_logic;
    signal select_Ia       : std_logic;
       
    signal empty           : std_logic_vector(NBITS-1 downto 0) := (others => '0');     
    signal empty_1         : std_logic_vector(NBITS-1 downto 0); 
    signal empty_2         : std_logic_vector(NBITS-1 downto 0); 
    signal empty_3         : std_logic_vector(NBITS-1 downto 0); 
    signal empty_4         : std_logic_vector(NBITS-1 downto 0); 
    signal empty_5         : std_logic_vector(NBITS-1 downto 0);
    signal empty_6         : std_logic_vector(NBITS-1 downto 0);
    signal temp_R          : std_logic_vector(8-1 downto 0);
        
    signal RGB_internal    : std_logic_vector(NBITS-1 downto 0);
    signal Ia_global       : std_logic_vector(NBITS-1 downto 0);
    signal cntrl_RGB       : std_logic_vector(1 downto 0);
    
    
begin  -- behavior

-- Temporary signal because of the parameter-calulations.
    output_R         <=   temp_R;
    
-- Initializing the components
    RGB_mux_in      :   RGB_mux        port map(R, G, B, Ia_g_R, Ia_g_G, Ia_g_B, cntrl_RGB, clk, RGB_internal, Ia_global); 
    Luminance_gen   :   luminance      port map(R, G, B, Luminance_value);
    signal_path     :   signal_pipe    port map(RGB_internal, Luminance_value, f, a, c, m, Ia_global, clk, Ia_1, Ia_2, Div_1, Div_2, 
                         select_div, select_Ia, Sigma_1, Sigma_2, select_sig, Ia_out, Sigma_out, temp_R);
    Parameters      :   param          port map(clk, reset, comp_phase, go, R, G, B, R_enable, G_enable, B_enable, Luminance_value, 
                         Lum_enable, m_in, cntrl_m, f_user_in, cntrl_f, temp_R, Sigma_out, Ia_out, select_sig, 
                         select_div, select_Ia, Ia_1, Ia_2, Sigma_1, Sigma_2, Div_1, Div_2, Ia_g_R, Ia_g_G, Ia_g_B, m, f);
    Control_unit    :   control_small  port map(clk, reset_in, go, reset, R_enable, G_enable, B_enable, Lum_enable, comp_phase, cntrl_RGB); 
        
end architecture behavior;