---------------------------------------------------------------
-- Testbench for the Ia module
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;

---------------------------------------------------------------
-- Entity
---------------------------------------------------------------

entity Ia_tb is
end entity Ia_tb;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavior of Ia_tb is

-- Generic constant    
    constant NBITS : natural := 18;

-- Defining the DUT     
component Ia_add
   generic(NBITS : natural := NBITS);
    port (A     : in  std_logic_vector (NBITS-1 downto 0);
          B     : in  std_logic_vector (NBITS-1 downto 0);
          c     : in  std_logic_vector (2 downto 0);
          output: out std_logic_vector (NBITS-1 downto 0)
          );
end component;
    
    signal A     : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal B     : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal c     : std_logic_vector(2 downto 0)       := (others=>'0');
    signal output: std_logic_vector(NBITS-1 downto 0);    
begin  
    
-- Initializing the DUT  
Ia_DUT: component Ia_add port map(A, B, c, output);

-- The test      
test: process
begin
    wait for 10 ns;
-- Test loop. Set the loop-borders to fit your test
   for c_loop in 0 to 7 loop
      for a_loop in 0 to 63 loop
         for b_loop in 0 to 63 loop
             A     <= conv_std_logic_vector(a_loop, NBITS);
             B     <= conv_std_logic_vector(b_loop, NBITS);
             c     <= conv_std_logic_vector(c_loop, 3);
             wait for 2 ns;
         end loop;
       end loop;
   end loop;
-- End the test   
   assert(false) severity error;
end process test;

-- Process that checks the answer and calculates the error
check_ans: process(output, A, B, c)
   variable temp           : real;
   variable temp_floor     : real;   
   variable error_value    : real;
   variable error_percent  : real;
begin
-- Calculates the ideal output-
    temp          := (real(conv_integer(A))*real(conv_integer(c))/real(8)) + (real(conv_integer(B))*(real(1)-(real(conv_integer(c))/real(8))));
-- Floors the ideal output    
    temp_floor    := floor(temp);
-- Calculates the error value
    error_value   := temp_floor - real(conv_integer(output));
-- Calculates the error percentage    
    if( NOT (temp_floor = real(0))) then
      error_percent := real(100)*error_value/temp_floor;
    else
      error_percent := real(0);
    end if;    
end process check_ans;

end architecture behavior;