---------------------------------------------------------------
-- Testbench for Generic N-bit adder
-- Source is taken from [Maryland] 
-- (http://www.csee.umbc.edu/help/VHDL/samples/add_g.vhdl)
-- with just minor corrections to fit this project.
---------------------------------------------------------------
library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- sample package declaration
package my_pkg is 
   function integer_to_slv(invect, bits : integer) return std_logic_vector;
end package my_pkg;

-- corresponding package body   
package body my_pkg is 
function integer_to_slv (invect, bits : integer)
   return std_logic_vector is
   
   variable n : std_logic_vector(bits-1 downto 0) := (others => '0');
   variable temp : integer := invect;
   
   begin

   for i in n'reverse_range loop
      if temp mod 2 = 1 then
         n(i) := '1';
      end if;
      temp := temp / 2;
   end loop;
   return n;
end function integer_to_slv;
end package body my_pkg;

-- Libaries
library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use WORK.my_pkg.integer_to_slv;

---------------------------------------------------------------
-- Entity
---------------------------------------------------------------

entity add_g_tb is end add_g_tb;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture test of add_g_tb is

-- Defining the DUT    
component add_g
   generic(left : natural :=17 );
   port 
      (a     : in std_logic_vector(17 downto 0);
       b     : in std_logic_vector(17 downto 0);
       cin   : in std_logic;
       sum   : out std_logic_vector(17 downto 0);
       cout  : out std_logic
   );
end component;

signal x, y: std_logic_vector(17 downto 0);
signal c_in, c_out: std_logic;
signal z: std_logic_vector(17 downto 0);
signal total_out : std_logic_vector(18 downto 0);

begin
-- Initializing the DUT    
DUT: add_g port map(x, y, c_in, z, c_out);

-- Test-process
testing: process
begin
   wait for 5 ns;
   
-- resetting    
   x <= (others => '0');
   c_in <= '0';
   wait for 5 ns;

-- Test-loop   
   for i in 0 to 262143 loop
      for j in 0 to 262143 loop
      x <= integer_to_slv(i, 18);
      wait for 1 ns;
      y <= integer_to_slv(j, 18);
      wait for 1 ns;
      total_out(18) <= c_out;
      total_out(17 downto 0) <= z(17 downto 0);  
      wait for 2 ns;       

-- Check answer      
      if(total_out /= (i+j)) then
         ASSERT (FALSE) SEVERITY ERROR;
      end if;
      wait for 1 ns;
      end loop;
   end loop;   
end process testing;

end test;