---------------------------------------------------------------
-- Testbench for the Adder/Subtractor from [Deschamps06]
-- with minor adjustments to fit this project
-- This module is fully generic
---------------------------------------------------------------
library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- sample package declaration
package my_pkg is 
   function integer_to_slv(invect, bits : integer) return std_logic_vector;
end package my_pkg;

-- corresponding package body  
package body my_pkg is  
function integer_to_slv (invect, bits : integer)
   return std_logic_vector is
   
   variable n : std_logic_vector(bits-1 downto 0) := (others => '0');
   variable temp : integer := invect;
   
   begin

   for i in n'reverse_range loop
      if temp mod 2 = 1 then
         n(i) := '1';
      end if;
      temp := temp / 2;
   end loop;
   return n;
end function integer_to_slv;
end package body my_pkg;

-- Libaries
library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_signed.all;
use WORK.my_pkg.integer_to_slv;

---------------------------------------------------------------
-- Entity
---------------------------------------------------------------

entity addsub_tb is end addsub_tb;


---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture test of addsub_tb is

-- Defining the DUT      
component addsub
   generic(NBITS : natural := 18);
   port 
      (x      : in std_logic_vector(17 downto 0);
       y      : in std_logic_vector(17 downto 0);
       control: in std_logic;       
       d_in   : in std_logic;
       z      : out std_logic_vector(17 downto 0);
       d_out  : out std_logic
   );
end component;

signal x       : std_logic_vector(17 downto 0);
signal y       : std_logic_vector(17 downto 0);
signal d_in    : std_logic;
signal d_out   : std_logic;
signal control : std_logic;
signal z       : std_logic_vector(17 downto 0);
signal total_out:std_logic_vector(18 downto 0);

begin
-- Initializing the DUT     
DUT: addsub port map(x, y, control, d_in, z, d_out);

-- Test-process
testing: process
begin
-- resetting 
   x <= (others => '0');
   y <= (others => '0');      
   d_in <= '0';
   control <= '0';   
-- Testing the adder: out = x+y   
   for i in 0 to 262143 loop
      for j in 0 to 262143 loop
      x <= integer_to_slv(i, 18);
      y <= integer_to_slv(j, 18);
      wait for 1 ns;         
      total_out(18) <= d_out;
      total_out(17 downto 0) <= z(17 downto 0);
      wait for 1 ns;               
      if(signed(total_out) /= (i+j)) then
         ASSERT (FALSE) SEVERITY ERROR;
      end if;
      end loop;
   end loop;   
   
-- resetting 
   x <= (others => '0');
   y <= (others => '0');   
   d_in <= '0';
   control <= '1';   
-- Testing the sub: out = x-y     
   for i in 0 to 262143 loop
      for j in 0 to 262143 loop
      x <= integer_to_slv(i, 18);
      y <= integer_to_slv(j, 18);
      wait for 1 ns;         
      total_out(18) <= d_out;
      total_out(17 downto 0) <= z(17 downto 0);
      wait for 1 ns;               
      if(signed(total_out) /= (i-j)) then
         ASSERT (FALSE) SEVERITY ERROR;
      end if;
      end loop;
   end loop; 
     
-- resetting 
   x <= (others => '0');
   y <= (others => '0');   
   d_in <= '1';
   control <= '0';   
-- Testing add with carry in: out = x+y+1     
   for i in 0 to 262143 loop
      for j in 0 to 262143 loop
      x <= integer_to_slv(i, 18);
      y <= integer_to_slv(j, 18);
      wait for 1 ns;         
      total_out(18) <= d_out;
      total_out(17 downto 0) <= z(17 downto 0);
      wait for 1 ns;               
      if(signed(total_out) /= (i+j+1)) then
         ASSERT (FALSE) SEVERITY ERROR;
      end if;
      end loop;
   end loop;   

-- resetting 
   x <= (others => '0');
   y <= (others => '0');   
   d_in <= '1';
   control <= '1';   
-- Testing sub with carry in: out = x-y-1    
   for i in 0 to 262143 loop
      for j in 0 to 262143 loop
      x <= integer_to_slv(i, 18);
      y <= integer_to_slv(j, 18);
      wait for 1 ns;         
      total_out(18) <= d_out;
      total_out(17 downto 0) <= z(17 downto 0);
      wait for 1 ns;               
      if(signed(total_out) /= (i-j-1)) then
         ASSERT (FALSE) SEVERITY ERROR;
      end if;
      end loop;
   end loop;   
   
end process testing;

end test;