---------------------------------------------------------------
-- Testbench for the avg module
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;

---------------------------------------------------------------
-- Entity
---------------------------------------------------------------

entity avg_tb is
end entity avg_tb;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavior of avg_tb is

-- Generic constant    
    constant NBITS : natural := 18;

-- Defining the DUT      
component avg is
    generic(NBITS : natural :=NBITS);
    port(
       input   :   in std_logic_vector(NBITS-1 downto 0);
       enable  :   in std_logic;
       reset   :   in std_logic;
       clk     :   in std_logic;
       output  :   out std_logic_vector(NBITS-1 downto 0));
end component avg;

    signal input   : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal clk     : std_logic                          := '0';
    signal enable  : std_logic                          := '0';    
    signal reset   : std_logic                          := '0';    
    signal output  : std_logic_vector(NBITS-1 downto 0);    
    

begin  
-- Initializing the DUT  
avg_DUT: component avg port map(input, enable, reset, clk, output);

-- Clock generation
clocking: process
   variable count : real :=real(1);
begin
    count := count + real(1);
    clk <= not clk;
    wait for 5 ns;
end process clocking;

-- The test    
test: process
begin
-- Resetting    
    reset <= '1';
    enable <= '0';
    wait for 20 ns;
    reset <= '0';
    wait for 2 ns;
-- Starting to send samples without enabling the avg.    
   for a_loop in 0 to 3072 loop
      if(a_loop = 350) then
-- Enabling the avg module to test the enable-function          
          enable <= '1';
      end if;
      input   <= conv_std_logic_vector(a_loop, NBITS);
      wait for 10 ns;
   end loop;

-- Resetting the avg-module in the middle of a stream to check the reset.
   reset <= '1';

   for a_loop in 0 to 256 loop
      input   <= conv_std_logic_vector(a_loop, NBITS);
      wait for 10 ns;
   end loop;

   reset <= '0';

-- A fully test-loop to check the avg-module
   for a_loop in 0 to (307200 + 1024) loop
      input   <= conv_std_logic_vector(a_loop, NBITS);
      wait for 10 ns;
   end loop;
   
-- Ending the test.   
   assert(false) severity error;
end process test;

-- Process that checks the estimation error.
check_ans: process(input, output, clk)
-- Defining the internal variables
   variable real_input        : real;
   variable real_output       : real;   
   variable real_avg          : real;   
   variable real_sum          : real;
   variable real_it           : real;         
   variable error_value       : real;
   variable error_percent     : real;
   variable real_counter      : real;
   variable internal_sum_1    : real;
   variable internal_sum      : real;
   variable checking          : real;
   variable checking_discrete : real;
   variable divide            : real;     
   
begin
   real_input      := real(conv_integer(input));
   real_output     := real(conv_integer(output));
-- Resets the internal variables.   
   if(rising_edge(clk)) then
       if(reset = '1') then
          real_it       := real(0);
          real_sum      := real(0);
          real_counter  := real(0);
          internal_sum  := real(0);
          internal_sum_1:= real(0);
       else
-- Keeps track of the ideal first partial sum       
          real_it       := real_it + real(1);
          real_sum      := real_sum + real_input;
          if(real_counter = real(1023)) then
-- Keeps track of the ideal partial sum after the first adder.              
              real_counter := real(0);
              internal_sum_1 := internal_sum_1 + floor(internal_sum/real(1024));              
              internal_sum   := real(0);
          else
              real_counter := real_counter + real(1);
              internal_sum := internal_sum + real_input; 
          end if;
              
       end if;
   end if;
-- Keeps track of the ideal average.
   checking          := internal_sum_1/real(300);
-- Keeps track of the ideal, discrete average with the divide-approximation implemented in the avg-circuit.
   divide            := real(1)/real(512)+real(1)/real(1024)+real(1)/real(4096)+real(1)/real(8192)+real(1)/real(16384);
   checking_discrete := internal_sum_1*divide;
-- Keeps track of the estimation-error.   
   if(real_it > real(0)) then
      real_avg        := real_sum/real_it;
      error_value     := real_output - real_avg;
   end if;
-- Calculates the estimation error percent
   if(real_output>real(0)) then
      error_percent  := abs(error_value/real_output);
   else
      error_percent := real(0);
   end if;
end process check_ans;

end architecture behavior; 