---------------------------------------------------------------
-- Testbench for the comp module
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;

---------------------------------------------------------------
-- Entity
---------------------------------------------------------------

entity comp_tb is
end entity comp_tb;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavior of comp_tb is
    
    constant NBITS : natural := 18;

-- Defining the DUT      
component comp is
    generic(NBITS : natural :=18);
    port(
        A           :   in std_logic_vector(NBITS-1 downto 0);
        B           :   in std_logic_vector(NBITS-1 downto 0);
        sel_input   :   in std_logic; 
        ----
        input       :   in std_logic_vector(NBITS-1 downto 0);
        sigma       :   in std_logic_vector(NBITS-1 downto 0);
        clk         :   in std_logic;
        output      :   out std_logic_vector(7 downto 0)
        );
end component comp;

    
    signal A        : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal B        : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal input    : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal sigma    : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    
    signal clk      : std_logic                          := '0';
    signal sel_input: std_logic                          := '0';
    signal output   : std_logic_vector(8-1 downto 0);    
begin  
-- Initializing the DUT  
comp_DUT: component comp port map(A, B, sel_input, input, sigma, clk, output);

-- Clock generation
process
begin
    wait for 5 ns;
    clk <= NOT clk;
end process;

-- The test     
test: process
begin
-- Testing the kompression    
    wait for 2 ns;
      for a_loop in 1 to 256 loop
         for b_loop in 1 to 256 loop
             input <= conv_std_logic_vector(a_loop, NBITS);
             sigma <= conv_std_logic_vector(b_loop, NBITS);
             wait for 10 ns;
         end loop;
       end loop;
    wait for 100 ns;
-- Testing the out=A/B - part of the comp-module    
    sel_input <= '1';
   for c_loop in 1 to 3 loop
      for a_loop in 1 to 256 loop
         for b_loop in a_loop to 256 loop
             input <= conv_std_logic_vector(c_loop, NBITS);
             A     <= conv_std_logic_vector(a_loop, NBITS);
             B     <= conv_std_logic_vector(b_loop, NBITS);
             wait for 10 ns;
         end loop;
       end loop;
   end loop;
-- Ending the test   
   assert(false) severity error;
end process test;

-- Process that checks the estimation error
check_ans: process(output, A, B, input, sigma, clk)
-- Defining the internal variables
   variable temp           : real;
   variable temp_floor     : real;
   variable tmp_1          : real;
   variable tmp_2          : real := real(0);
   variable real_A         : real;
   variable real_B         : real;
   variable real_input     : real;
   variable real_sigma     : real;
   variable real_output    : real;
   variable error_value    : real;
   variable error_percent  : real;
begin
-- Setting the internal variable-inputs    
    real_A        := real(conv_integer(A));
    real_B        := real(conv_integer(B));
    real_input    := real(conv_integer(input));
    real_sigma    := real(conv_integer(sigma));
    real_output   := real(conv_integer(output))/real(256);

-- Calculating the ideal compressed value
    if(NOT ((real_input+real_sigma)=real(0))) then
       temp          := real_input / (real_input + real_sigma);
    end if;

-- Calculating the ideal A/B - value
    if(sel_input='1' AND NOT(real_B = real(0))) then
        temp := real_A/(real_B);
    end if;
    
-- Pipelining    
    if(rising_edge(clk)) then
        tmp_2 := tmp_1;
        tmp_1 := temp;
    end if;    
    
-- Calculating the error        
    error_value := real_output - tmp_2;

-- Calculating the error percentage       
    if( (tmp_2 > real(0))) then
      error_percent := real(100)*error_value/tmp_2;
    else
      error_percent := real(0);
    end if;
end process check_ans;
end architecture behavior;