---------------------------------------------------------------
-- Testbench for the f_inn module
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;

---------------------------------------------------------------
-- Entity
---------------------------------------------------------------

entity f_inn_tb is
end entity f_inn_tb;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavior of f_inn_tb is

-- Generic constant    
    constant NBITS : natural := 18;

-- Defining the DUT       
component f_inn
	 generic(NBITS : natural := NBITS);
    port (input    : in  std_logic_vector (NBITS-1 downto 0);
          f_user   : in  std_logic_vector (NBITS-1 downto 0);
          cntrl_f  : in  std_logic;
          output   : out std_logic_vector (NBITS-1 downto 0)
          );
end component;
    
    signal input    : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal output   : std_logic_vector(NBITS-1 downto 0);    
    signal f_user   : std_logic_vector(NBITS-1 downto 0) := (others=>'0');  
    signal cntrl_f  : std_logic := '0';          

begin  

-- Initializing the DUT  
f_inn_DUT: component f_inn port map(input, f_user, cntrl_f, output);

-- The test     
test: process
begin
    wait for 10 ns;
-- Test loop. Testing out = 1.375*input + f_user    
   for a_loop in 0 to 262144 loop
      for b_loop in 0 to 262144 loop
          input     <= conv_std_logic_vector(a_loop, NBITS);
          f_user    <= conv_std_logic_vector(b_loop, NBITS);
          wait for 5 ns;
      end loop;
   end loop;

-- Switching from add to sub   
   cntrl_f   <= NOT cntrl_f;

-- Test loop. Testing out = 1.375*input - f_user   
   for a_loop in 0 to 262144 loop
      for b_loop in 0 to a_loop loop
          input     <= conv_std_logic_vector(a_loop, NBITS);
          f_user    <= conv_std_logic_vector(b_loop, NBITS);
          wait for 5 ns;
      end loop;
   end loop;   
-- Ending the test   
   assert(false) severity error;
end process test;

-- Process that checks the error.
check_ans: process(output, input)
   variable temp           : real;
   variable temp_floor     : real;   
   variable error_value    : real;
begin
-- Calculates the ideal output
       if(cntrl_f='0') then
         temp          := real(1.375)*(real(conv_integer(input))) + real(conv_integer(f_user));    
       else
         temp          := real(1.375)*(real(conv_integer(input))) - real(conv_integer(f_user));    
       end if;
-- Floors the ideal output
    temp_floor    := floor(temp);
-- Calculates the error     
    error_value   := temp_floor - real(conv_integer(output));
end process check_ans;
end architecture behavior; 