---------------------------------------------------------------
-- Testbench for the luminance module
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;

---------------------------------------------------------------
-- Entity
---------------------------------------------------------------

entity luminance_tb is
end entity luminance_tb;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavior of luminance_tb is

-- Generic constant        
    constant NBITS : natural := 18;

-- Defining the DUT    
component luminance
   generic(NBITS : natural :=18 );
    port (red   : in  std_logic_vector (NBITS-1 downto 0);
          green : in  std_logic_vector (NBITS-1 downto 0);
          blue  : in  std_logic_vector (NBITS-1 downto 0);
          output: out std_logic_vector (NBITS-1 downto 0)
          );
end component;
    
    signal red   : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal green : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal blue  : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal output: std_logic_vector(NBITS-1 downto 0);    
    
    signal clk   : std_logic                          := '0';
begin  
    
-- Initializing the DUT  
lum_DUT: component luminance port map(red, green, blue, output);

-- The test     
test: process
begin
    wait for 10 ns;
-- Test loop. Set the loop-borders to fit your test    
   for r in 0 to 1000 loop
      for g in 0 to 1000 loop
         for b in 0 to 100000 loop
             red   <= conv_std_logic_vector(r*100, NBITS);
             green <= conv_std_logic_vector(g*100, NBITS);
             blue  <= conv_std_logic_vector(b, NBITS);
             wait for 2 ns;
         end loop;
       end loop;
   end loop;
-- End the test    
   assert(false) severity error;
end process test;

-- Clock generation
process
begin
    wait for 1 ns;
    clk <= not clk;    
end process;

-- Process that checks the answer and calculates the error
check_ans: process(output, red, green, blue, clk)
   variable temp           : real;
   variable error          : real;
   variable error_percent  : real;
   variable iteration      : real := real(0);
   variable error_sum      : real := real(0);
   variable error_avg      : real := real(0);
begin
-- Calculates the ideal output    
    temp          := (real(conv_integer(red))*0.2125) + (0.7154*real(conv_integer(green))) + (0.0721*real(conv_integer(blue)));
-- Calculates the error value    
    error         := temp - real(conv_integer(output));
-- Calculates the error percentage    
    if(temp>real(0)) then
       error_percent := abs(error * real(100)/temp);
    end if;
-- Calculates the average error    
    if(rising_edge(clk)) then
          iteration := iteration+real(1);
          error_sum := error_sum + abs(error);     
    end if;
    if(iteration>real(0)) then
        error_avg := error_sum/iteration;
    end if;
end process check_ans;
end architecture behavior;  -- of maksimum_tb