---------------------------------------------------------------
-- Testbench for the sigma module
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;

---------------------------------------------------------------
-- Entity
---------------------------------------------------------------

entity sigma_tb is
end entity sigma_tb;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavior of sigma_tb is

-- Generic constant      
    constant NBITS : natural := 18;

-- Defining the DUT     
component sigma
    generic(NBITS : natural :=18);
    port(
       A       :   in std_logic_vector(NBITS-1 downto 0);
       B       :   in std_logic_vector(NBITS-1 downto 0);
       use_last:   in std_logic;        
       f       :   in std_logic_vector(NBITS-1 downto 0);
       Ia      :   in std_logic_vector(NBITS-1 downto 0);
       m       :   in std_logic_vector(12 downto 0);
       clk     :   in std_logic;
       output  :   out std_logic_vector(NBITS-1 downto 0));
end component;

    signal A       : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal B       : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal f       : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal Ia      : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal m       : std_logic_vector(12 downto 0) := (others=>'0');        
    signal clk     : std_logic                          := '0';
    signal use_last: std_logic                          := '0';    
    signal output  : std_logic_vector(NBITS-1 downto 0);    

begin  
    
-- Initializing the DUT  
sigma_DUT: component sigma port map(A, B, use_last, f, Ia, m, clk, output);

-- Clock generation
clocking: process
begin
    clk <= not clk;
    wait for 5 ns;
end process clocking;

-- The test
test: process
begin
    wait for 2 ns;
-- Test loop. Testing the sigma-calulation. Set the loop-borders to fit your test        
   for a_loop in 0 to 511 loop
      for b_loop in 0 to 511 loop
         for c_loop in 0 to 32 loop
             f     <= conv_std_logic_vector(a_loop*512, NBITS);
             Ia    <= conv_std_logic_vector(b_loop*512, NBITS);
             m     <= conv_std_logic_vector(c_loop*256, 13);
             wait for 10 ns;
         end loop;
      end loop;
   end loop;

-- Switches the mux.
   use_last <= '1';
   
-- Test loop. Testing the out=2^(A*B)-calulation. Set the loop-borders to fit your test        
   for a_loop in 0 to 256 loop
      for b_loop in 0 to 256 loop
          A     <= conv_std_logic_vector(a_loop, NBITS);
          B     <= conv_std_logic_vector(b_loop, NBITS);
          wait for 10 ns;
      end loop;
   end loop;
-- Ending the test
   assert(false) severity error;
end process test;

-- Calculating the idael output and the error.
check_ans: process(A, B, use_last, f, Ia, m, output, clk)
   variable real_f            : real := real(0);
   variable real_Ia           : real := real(0);
   variable real_Ia_log       : real := real(0);
   variable real_m            : real := real(0);
   variable real_m_2          : real := real(0);   
   variable real_output       : real := real(0);
   variable vhdl_output       : real := real(0);
   variable real_temp_1       : real := real(0);   
   variable real_temp_2       : real := real(0);      
   variable error_value       : real := real(0);
   variable error_percent     : real := real(0);
begin
-- Checks the output-value    
   vhdl_output       := real(conv_integer(output));    
-- Calculats the real input parameters.    
   real_f         := real(conv_integer(f)) / real(2097152);
   real_Ia        := real(conv_integer(Ia)) ;
   real_m            := real(conv_integer(m)) / real(8192);   
-- Calculats the ideal base 2 logarithm of Ia   
   if(real_Ia>real(0)) then       
      real_Ia_log    := LOG(real_Ia)/LOG(real(2));
   end if;
-- Calulates the ideal    "f+log2(Ia)"
   real_temp_1       := real_f + real_Ia_log;
-- pipelining   
   if(rising_edge(clk)) then
       real_temp_2 := real_temp_1;
       real_m_2    := real_m;
   end if;
-- Calculates the ideal output   
   if(use_last = '0') then
      real_output := ((real(2)**(real_temp_2*real_m_2)));
   else
      real_output := ((real(2)**(real(conv_integer(A))*real(conv_integer(B)))));
   end if;
-- Calculates the error value     
   error_value := real_output - vhdl_output;
-- Calculates the error percentage   
   if(real_output> real(0)) then
       error_percent := error_value/real_output;
   end if;
end process check_ans;

end architecture behavior; 