---------------------------------------------------------------
-- Testbench for the signal_pipe module
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.math_real.all;

---------------------------------------------------------------
-- Entity
---------------------------------------------------------------

entity signal_pipe_tb is
end entity signal_pipe_tb;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavior of signal_pipe_tb is

-- Generic constant    
    constant NBITS : natural := 18;

-- Defining the DUT     
component signal_pipe is
  generic(NBITS  : natural := NBITS         -- # bits
          );
    port (Input_RGB   : in  std_logic_vector (NBITS-1 downto 0);
    			 Input_Lum   : in  std_logic_vector (NBITS-1 downto 0);          
          f           : in  std_logic_vector (NBITS-1 downto 0);
          a           : in  std_logic_vector (2 downto 0);
          c           : in  std_logic_vector (2 downto 0);          
          m           : in  std_logic_vector (12 downto 0);          
          Ia_global   : in  std_logic_vector (NBITS-1 downto 0);
          clk         : in  std_logic;
          Ia_in_1     : in  std_logic_vector (NBITS-1 downto 0);
          Ia_in_2     : in  std_logic_vector (NBITS-1 downto 0);  
          Div_1       : in  std_logic_vector (NBITS-1 downto 0);
          Div_2       : in  std_logic_vector (NBITS-1 downto 0);
          select_div  : in  std_logic;
          select_Ia   : in  std_logic;
          Sigma_1     : in  std_logic_vector (NBITS-1 downto 0);
          Sigma_2     : in  std_logic_vector (NBITS-1 downto 0);
          select_sig  : in  std_logic;
          Ia_out      : out std_logic_vector (NBITS-1 downto 0);
          Sigma_out   : out std_logic_vector (NBITS-1 downto 0);
          
          output      : out std_logic_vector (8-1 downto 0)
          );
end component signal_pipe;

-- Defining aditional component
component luminance is
  generic(NBITS  : natural := 18         -- # bits
          );
    port (red   : in  std_logic_vector (NBITS-1 downto 0);
          green : in  std_logic_vector (NBITS-1 downto 0);
          blue  : in  std_logic_vector (NBITS-1 downto 0);
          output: out std_logic_vector (NBITS-1 downto 0)
          );
end component luminance;

    signal Input_RGB       : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal Input_Lum       : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal f               : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal a               : std_logic_vector(2 downto 0)       := (others=>'0');    
    signal c               : std_logic_vector(2 downto 0)       := (others=>'0');
    signal m               : std_logic_vector(12 downto 0)      := (others=>'0');        
    signal Ia_global       : std_logic_vector(NBITS-1 downto 0) := (others=>'0');
    signal clk             : std_logic                          := '0';    
    signal Div_1           : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal Div_2           : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal select_div      : std_logic                          := '0';    
    signal Sigma_1         : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal Sigma_2         : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal select_sig      : std_logic                          := '0'; 
    signal select_Ia       : std_logic                          := '0';    
    signal Ia_in_1         : std_logic_vector(NBITS-1 downto 0);    
    signal Ia_in_2         : std_logic_vector(NBITS-1 downto 0);    
    signal Ia_out          : std_logic_vector(NBITS-1 downto 0);    
    signal Sigma_out       : std_logic_vector(NBITS-1 downto 0);    
    signal output          : std_logic_vector(8-1 downto 0);    

    signal R               : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal G               : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    
    signal B               : std_logic_vector(NBITS-1 downto 0) := (others=>'0');    

begin 

-- Initializing the components
signal_pipe_DUT: component signal_pipe port map(Input_RGB, Input_Lum, f, a, c, m, Ia_global, clk, 
                                                Ia_in_1, Ia_in_2, Div_1, Div_2, select_div, select_Ia, Sigma_1, Sigma_2, select_sig,
                                                Ia_out, Sigma_out, output);
Lum:             component luminance   port map(R, G, B, Input_Lum);

-- Clock generation
clocking: process
begin
    clk <= not clk;
    wait for 5 ns;
end process clocking;

-- The test
test: process
begin

-- Setting the compressionparameters. Adjust this values for different tests.
    f          <= "000110000000000000";   
    c          <= "000";
    a          <= "111";
    m          <= "1010110001000";        
    Ia_global  <= "100001101010001110";   
    wait for 2 ns;
-- The test-loop. Testing with regular values, and not a real picture.
   for a_loop in 0 to 511 loop
      for b_loop in 0 to 511 loop
         for c_loop in 0 to 512 loop
-- Setting each color         
                R     <= conv_std_logic_vector(a_loop*512, NBITS);
                G     <= conv_std_logic_vector(b_loop*512, NBITS);
                B     <= conv_std_logic_vector(c_loop*512, NBITS);
            for d_loop in 0 to 2 loop  
-- Selecting the color to compress.
               case d_loop is
                  when 0 => Input_RGB <= conv_std_logic_vector(a_loop*512, NBITS);
                  when 1 => Input_RGB <= conv_std_logic_vector(b_loop*512, NBITS);
                  when 2 => Input_RGB <= conv_std_logic_vector(c_loop*512, NBITS);
                  when others => Input_RGB <= (others => '0');
               end case;
               wait for 10 ns;
            end loop;
         end loop;
      end loop;
   end loop;
-- Ending the test
   assert(false) severity error;
end process test;

-- Calculating the idael output and the error.
check_ans: process(R, G, B, Input_Lum, a, f, c, m, Ia_global, output, clk)
   variable real_R            : real;
   variable real_G            : real;   
   variable real_B            : real;   
   variable real_RGB          : real;  
   variable real_RGB_1        : real;  
   variable real_RGB_2        : real;  
   variable real_RGB_3        : real;      
   variable real_RGB_4        : real;    
   variable real_RGB_5        : real;    
   variable real_Lum          : real;
   variable real_a            : real;
   variable real_f            : real;
   variable real_c            : real;   
   variable real_m            : real;
   variable real_Ia_global    : real;
   variable real_output       : real;
   variable real_output_2     : real;
   variable vhdl_output       : real;
   
   variable real_Ia_local     : real;
   variable vhdl_Ia_local     : real;
   variable real_Ia           : real;
   variable real_Ia_2         : real;   
   variable real_Ia_3         : real;    
   variable vhdl_Ia           : real;
   variable real_sigma        : real;
   variable real_sigma_1      : real;
   variable real_sigma_2      : real;
   variable real_sigma_3      : real;
   variable vhdl_sigma        : real;
   
   variable temp_1            : real;   
   variable temp_2            : real;  

   variable temp_floor        : real;   
   variable error_value       : real;
   variable error_percent     : real;
   variable error_too_high    : boolean;
   variable worst_error       : real := real(0) ;
begin
-- Calulating the real value of the input signals    
   real_R         := real(conv_integer(R));
   real_G         := real(conv_integer(G));
   real_B         := real(conv_integer(B));
   real_RGB       := real(conv_integer(Input_RGB));
   real_Lum       := real(conv_integer(Input_Lum));
   real_a         := real(conv_integer(a)) / real(7);
   real_c         := real(conv_integer(c)) / real(7);
   real_f         := real(conv_integer(f)) / real(8192);
   real_m         := real(conv_integer(m)) / real(8192);
   vhdl_output    := real(conv_integer(output));
   real_Ia_global := real(conv_integer(Ia_global));

-- Calulating the Intensity Adaption Level
   real_Ia_local  := real_c*real_RGB + (real(1)-real_c)*real_Lum;
   real_Ia        := real_a*real_Ia_local + (real(1)-real_a)*real_Ia_global;  
   
-- Pipelining the signals
   if(rising_edge(clk)) then      
      real_Ia_3         := real_Ia_2;       
      real_Ia_2         := real_Ia;
      real_RGB_5        := real_RGB_4;      
      real_RGB_4        := real_RGB_3;
      real_RGB_3        := real_RGB_2;
      real_RGB_2        := real_RGB_1;
      real_RGB_1        := real_RGB;
      real_sigma_3      := real_sigma_2;
      real_sigma_2      := real_sigma_1;
      real_sigma_1      := real_sigma;
-- Calulating the ideal sigma      
      if(real_m>real(0) AND real_Ia_3>real(0)) then
         real_sigma   := ((2**(real_f))*real_Ia_3)**real_m;
      else
         real_sigma   := real(1);
      end if;      
-- Calculating the ideal output signal      
      if((real_RGB_5+real_sigma_3)>real(0)) then
         real_output    := floor(real(256)*real_RGB_5 / (real_RGB_5 + real_sigma_3));
      else
         real_output    := real(0);
      end if;           
   end if;
-- Calulating the error value
   error_value := real_output - vhdl_output;
-- Calulating the error percent   
   if(real_output > real(0)) then
       error_percent := real(100)*error_value/real_output;
   end if;

end process check_ans;

end architecture behavior; 