---------------------------------------------------------------
-- Error correction circuit for the Abed03 Logarithm module
-- with minor adjustments to fit this project
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

---------------------------------------------------------------
-- Defining the entity
---------------------------------------------------------------

entity abed03_2region_error_correction is
port(
      input   : in std_logic_vector(17 downto 0);
      output  : out std_logic_vector(17 downto 0)
      );
end abed03_2region_error_correction;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavioral of abed03_2region_error_correction is

-- Defining the components   

component full_adder   
    port(    
    A       : in std_logic;
    B       : in std_logic;
    C       : in std_logic;
    S       : out std_logic;
    C_out   : out std_logic
    );
end component full_adder;

component half_adder   
    port(    
    A       : in std_logic;
    B       : in std_logic;
    S       : out std_logic;
    C_out   : out std_logic
    );
end component half_adder;

   signal xor_11 : std_logic;
   signal xor_10 : std_logic;
      
   signal c_9    : std_logic;
   signal c_10   : std_logic;
   signal c_11   : std_logic;
   signal c_12   : std_logic;
   signal c_13   : std_logic;   
begin

-- Initializing the components

FA1: full_adder port map(input(9)  ,xor_11 ,c_9 ,output(9) ,c_10 );  
HA1: half_adder port map(input(8)  ,xor_10 ,output(8)  ,c_9 );
HA2: half_adder port map(input(10) ,c_10   ,output(10) ,c_11 );
HA3: half_adder port map(input(11) ,c_11   ,output(11) ,c_12 );
HA4: half_adder port map(input(12) ,c_12   ,output(12) ,c_13 );

-- connecting the internal signals
process(input)
begin
   output(17 downto 13) <= input(17 downto 13);
   output(7 downto 0)   <= input(7 downto 0); 
   xor_10   <= input(12) XOR input(10);
   xor_11   <= input(12) XOR input(11);      
end process;

end architecture behavioral;