---------------------------------------------------------------
-- Testbench for testing the Abed03-Logarithm unit with the 6-region Error correction circuit.
-- The output is logged in the txt-file picture_output.txt which may be subject to error-calculation 
-- with the Matlab function find_error.
---------------------------------------------------------------
library STD;
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_textio.all;
use STD.textio.all;

---------------------------------------------------------------
-- Defining the entity
---------------------------------------------------------------

entity Logarithm_Abed03_6region_tb is
end Logarithm_Abed03_6region_tb;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture circuits of Logarithm_Abed03_6region_tb is

-- Initializing the log-file     
      file   write_file : text  open write_mode is "Logarithm_Abed03_6region_tb_log.txt";    
    
      signal one        : std_logic_vector(17 downto 0) := "000000000000000001";
	  	signal evaluate   : std_logic;
 	 	signal clk        : std_logic := '0'; 
  		signal output     : std_logic_vector(17 downto 0);
  		signal input						: std_logic_vector(17 downto 0) := "000000000000000001";

-- Defining the DUT   
  component Logarithm_Abed03_6region 
    port(
       input   :   in std_logic_vector(17 downto 0);
       clk     :   in std_logic;
       output  :   out std_logic_vector(17 downto 0));
  end component Logarithm_Abed03_6region;

begin  
-- Initializing the DUT      
  Logarithm_Abed03_6region_testsubject:  Logarithm_Abed03_6region port map(input, clk, output); 

-- Clock generation
clocking:process
begin
    loop
       wait for 5 ns;
       clk <= not clk;
    end loop;
end process;

-- Logging both the input and the output of the module
logging:process(clk)
   variable my_line : STD.textio.line;
   variable lcntr : std_logic_vector(4 downto 0);
   alias swrite is write [line, string, side, width] ;
begin
   if(not(output="UUUUUUUUUUUUUUUUUU")) then
      write(my_line, input);
      swrite(my_line, " ");
      write(my_line, output);
      writeline(write_file, my_line);           
   end if;
end process logging;    

-- The test-sequence.
test:process 
 	begin
-- Resets the input signals 	  	    
		evaluate <= '1';
		input <= "000000000000000000";
		wait for 1 ns;
-- Test-loop			
   for i in 0 to 262142 loop
		input <= unsigned(input) + unsigned(one);
		wait for 5 ns;
	end loop;
-- Ending the simulation	
	assert false report "NONE. End of simulation." severity failure;
end process test;
end architecture circuits; -- of Logarithm_Abed03_6region_tb