---------------------------------------------------------------
-- 18x5 ROM acting like an 18-to-5 decoder. Submodule in the SanGregory99 Logarithm unit.
-- with minor adjustments to fit this project
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

---------------------------------------------------------------
-- Defining the entity
---------------------------------------------------------------

entity ROM18x5 is          	-- 
port(
    adress     : in std_logic_vector(17 downto 0);
    value      : out std_logic_vector(4 downto 0));
end ROM18x5;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavioral of ROM18x5 is
begin   

-- Process that decodes the 18-bit input and sets the output equal the position of the'1'.
-- InputSignals with more than one '1' is not valid.
process(adress)
begin
	case adress(17 downto 0) is
		when "000000000000000000" => value(4 downto 0) <= "00000";
		when "000000000000000001" => value(4 downto 0) <= "00000";
		when "000000000000000010" => value(4 downto 0) <= "00001";
		when "000000000000000100" => value(4 downto 0) <= "00010";
		when "000000000000001000" => value(4 downto 0) <= "00011";
		when "000000000000010000" => value(4 downto 0) <= "00100";
		when "000000000000100000" => value(4 downto 0) <= "00101";
		when "000000000001000000" => value(4 downto 0) <= "00110";
		when "000000000010000000" => value(4 downto 0) <= "00111";
	   when "000000000100000000" => value(4 downto 0) <= "01000";
	   when "000000001000000000" => value(4 downto 0) <= "01001";
	   when "000000010000000000" => value(4 downto 0) <= "01010";
	   when "000000100000000000" => value(4 downto 0) <= "01011";
	   when "000001000000000000" => value(4 downto 0) <= "01100";
	   when "000010000000000000" => value(4 downto 0) <= "01101";
	   when "000100000000000000" => value(4 downto 0) <= "01110";
	   when "001000000000000000" => value(4 downto 0) <= "01111";
	   when "010000000000000000" => value(4 downto 0) <= "10000";
	   when "100000000000000000" => value(4 downto 0) <= "10001";
		when others			            =>	value(4 downto 0) <= "11111";
	end case;	
end process;

end architecture behavioral; -- of ROM18x5
