---------------------------------------------------------------
-- Error correction circuit for the Abed03 Logarithm module
-- with minor adjustments to fit this project
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

---------------------------------------------------------------
-- Defining the entity
---------------------------------------------------------------

entity abed03_6region_error_correction is
port(
      input   : in std_logic_vector(17 downto 0);
      output  : out std_logic_vector(17 downto 0)
      );
end abed03_6region_error_correction;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavioral of abed03_6region_error_correction is
    
-- Defining the components       
    
component full_adder   
    port(    
    A       : in std_logic;
    B       : in std_logic;
    C       : in std_logic;
    S       : out std_logic;
    C_out   : out std_logic
    );
end component full_adder;

component half_adder   
    port(    
    A       : in std_logic;
    B       : in std_logic;
    S       : out std_logic;
    C_out   : out std_logic
    );
end component half_adder;

   signal xor_1 : std_logic;
   signal xor_2 : std_logic;
   signal xor_3  : std_logic;
   signal xor_4 : std_logic;
   signal xor_5 : std_logic;
   signal xor_6  : std_logic;

   signal and_1 : std_logic;
   signal and_2 : std_logic;

   signal or_1 : std_logic;
   signal or_2 : std_logic;

   signal mux_1 : std_logic;      
   signal mux_2 : std_logic;      
   signal mux_3 : std_logic;            
   signal mux_4 : std_logic;      
   signal mux_5 : std_logic;      
   signal mux_6 : std_logic;            

   signal a_1 : std_logic;      
   signal a_2 : std_logic;      
   signal a_3 : std_logic;            
   signal a_4 : std_logic;      
   signal a_5 : std_logic;      
   signal a_6 : std_logic;            
      
   signal c_1   : std_logic;   
   signal c_2   : std_logic;
   signal c_3   : std_logic;
   signal c_4   : std_logic;
   signal c_5   : std_logic;
   signal c_6   : std_logic;   
   signal c_7   : std_logic;   
   signal c_8   : std_logic;
   signal c_9   : std_logic;
   signal c_10  : std_logic;
   signal c_11  : std_logic;
   signal c_12  : std_logic;   
   signal c_13  : std_logic;   --dummy
   signal c_14  : std_logic;   --dummy
   
begin

-- Initializing the components

FA1:  full_adder port map(input(9)  ,mux_2 ,c_4 ,a_4 ,c_5 );  
FA2:  full_adder port map(input(8)  ,mux_3 ,c_3 ,a_5 ,c_4 );  
FA3:  full_adder port map(input(7)  ,mux_4 ,c_2 ,a_6 ,c_3 );  
FA4:  full_adder port map(input(6)  ,mux_5 ,c_1 ,output(6) ,c_2 );  

HA1:  half_adder port map(input(12) ,c_7 ,a_1 ,c_13 );
HA2:  half_adder port map(input(11) ,c_6 ,a_2 ,c_7 );
HA3:  half_adder port map(input(10) ,c_5 ,a_3 ,c_6 );
HA4:  half_adder port map(input(5)  ,mux_6  ,output(5) ,c_1 );
HA5:  half_adder port map(a_1 ,c_12 ,output(12)  ,c_14 );
HA6:  half_adder port map(a_2 ,c_11 ,output(11) ,c_12 );
HA7:  half_adder port map(a_3 ,c_10 ,output(10) ,c_11 );
HA8:  half_adder port map(a_4 ,c_9 ,output(9) ,c_10 );
HA9:  half_adder port map(a_5 ,c_8 ,output(8) ,c_9 );
HA10: half_adder port map(a_6 ,mux_1 ,output(7) ,c_8 );

-- connecting the internal signals
process(input, xor_1, xor_2, xor_3, xor_4, xor_5, xor_6, or_1, or_2, and_1, and_2)
begin
   output(17 downto 13) <= input(17 downto 13);
   output(4 downto 0)   <= input(4 downto 0); 
   
   xor_1   <= NOT (input(11) XOR input(10));      
   xor_2   <= input(12) XOR input(11);
   xor_3   <= input(12) XOR input(10);      
   xor_4   <= input(12) XOR input(9);      
   xor_5   <= input(12) XOR input(8);
   xor_6   <= input(12) XOR input(7);      
   
   and_1   <= (NOT input(12)) AND input(11) AND (NOT input(10));
   and_2   <= xor_1 AND xor_2;      

   or_1    <= input(12) OR input(11);
   or_2    <= input(10) OR input(9);
   
   mux_1   <= or_2 AND (NOT or_1) ;
   mux_2   <= xor_2 ;
   mux_3   <= (NOT xor_2) AND xor_3 ;
   mux_4   <= ((NOT xor_2) AND xor_4) OR (xor_2 AND and_2);
   mux_5   <= ((NOT xor_2) AND xor_5) OR xor_2;
   mux_6   <= ((NOT xor_2) AND xor_5) OR (xor_2 AND and_1);
   
end process;

end architecture behavioral;