---------------------------------------------------------------
-- Error correction circuit for the SanGregory99 Logarithm module
-- with minor adjustments to fit this project
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

---------------------------------------------------------------
-- Defining the entity
---------------------------------------------------------------

entity sangregory_error_correction is
port(
      input   : in std_logic_vector(17 downto 0);
      output  : out std_logic_vector(17 downto 0)
      );
end sangregory_error_correction;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavioral of sangregory_error_correction is
    
-- Defining the components    
    
component full_adder   
    port(    
    A       : in std_logic;
    B       : in std_logic;
    C       : in std_logic;
    S       : out std_logic;
    C_out   : out std_logic
    );
end component full_adder;

component half_adder   
    port(    
    A       : in std_logic;
    B       : in std_logic;
    S       : out std_logic;
    C_out   : out std_logic
    );
end component half_adder;

   signal xor_11 : std_logic;
   signal xor_10 : std_logic;
   signal xor_9  : std_logic;
   
   signal c_7    : std_logic;
   signal c_8    : std_logic;
   signal c_9    : std_logic;
   signal c_10   : std_logic;
   signal c_11   : std_logic;
   signal c_12   : std_logic;
begin

-- Initializing the components

FA1: full_adder port map(input(7)  ,xor_9 ,'0' ,output(7) ,c_7 );  
FA2: full_adder port map(input(8)  ,xor_10 ,c_7 ,output(8) ,c_8 );  
FA3: full_adder port map(input(9)  ,xor_11 ,c_8 ,output(9) ,c_9 );  
HA1: half_adder port map(input(10) ,c_9 ,output(10) ,c_10 );
HA2: half_adder port map(input(11) ,c_10 ,output(11) ,c_11 );
HA3: half_adder port map(input(12) ,c_11 ,output(12) ,c_12 );

-- connecting the internal signals
process(input)
begin
   output(17 downto 13) <= input(17 downto 13);
   output(6 downto 0)   <= input(6 downto 0); 
   xor_9    <= input(12) XOR input(9);
   xor_10   <= input(12) XOR input(10);
   xor_11   <= input(12) XOR input(11);      
end process;

end architecture behavioral;