---------------------------------------------------------------
-- Logarithm unit designed after the documentation in SanGregory99
-- with minor adjustments to fit this project
---------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

---------------------------------------------------------------
-- Defining the entity
---------------------------------------------------------------

entity Logarithm_SanGregory is
    port(
       input   :   in std_logic_vector(17 downto 0);
       evaluate:   in std_logic;
       output  :   out std_logic_vector(17 downto 0));
end entity Logarithm_SanGregory;

---------------------------------------------------------------
-- Architecture
---------------------------------------------------------------

architecture behavioral of Logarithm_SanGregory is
    
-- Defining the components
    
 component leading_one_18   
    port(    
    input     : in std_logic_vector(17 downto 0);
    eval      : in std_logic;
    shift     : out std_logic_vector(17 downto 0));
  end component leading_one_18;
  
  component ROM18x5
      port(
      adress  : in std_logic_vector(17 downto 0);
      value   : out std_logic_vector(4 downto 0)  );
  end component ROM18x5;
   
  component Shifter18 
      port(
      number_of_shifts   : in std_logic_vector(4 downto 0);
      input              : in std_logic_vector(17 downto 0);
      output             : out std_logic_vector(17 downto 0));
  end component Shifter18;
  
  component sangregory_error_correction 
      port(
      input              : in std_logic_vector(17 downto 0);
      output             : out std_logic_vector(17 downto 0));
  end component sangregory_error_correction;

	signal internal : std_logic_vector(17 downto 0);    
	signal shifted  : std_logic_vector(17 downto 0);    	
	signal LOD18    : std_logic_vector(17 downto 0);    
	signal LOD5     : std_logic_vector(4 downto 0);    
	signal temp_out : std_logic_vector(17 downto 0);    
begin

-- Initializing the components
LOD_circuit: leading_one_18              port map(internal, evaluate, LOD18);
ROM:         ROM18x5                     port map(LOD18, LOD5);
Shifter:     Shifter18                   port map(LOD5, internal, shifted);
err_corr:    sangregory_error_correction port map(temp_out, output);

-- connectiong the internal signals
no_clocking:process(input, LOD5, shifted)
begin
        if(evaluate ='1') then
           internal(17 downto 0)   <= input(17 downto 0);
           temp_out(17 downto 13)  <= LOD5(4 downto 0);
           temp_out(12 downto 0)   <= shifted(17 downto 5);
       end if;         
end process no_clocking;

end architecture;
